#
# Copyright 2019 by Delphix
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

#
# This class has been automatically generated from:
#     /delphix-oracle-base-attach-data.json
#
# Do not edit this file manually!
#

from __future__ import unicode_literals

from delphixpy.v1_10_0.web.objects.AttachData import AttachData
from delphixpy.v1_10_0 import factory
from delphixpy.v1_10_0 import common

class __Undef(object):
    def __repr__(self):
        return "undef"

    def __setattr__(self, name, value):
        raise Exception('Cannot modify attributes of __Undef.')

_UNDEFINED = __Undef()

try:
    TEXT_TYPE = unicode
except NameError:
    TEXT_TYPE = str

class OracleBaseAttachData(AttachData):
    """
    *(extends* :py:class:`v1_10_0.web.vo.AttachData` *)* Represents common
    parameters to attach all Oracle databases.
    """
    def __init__(self, undef_enabled=True):
        super(OracleBaseAttachData, self).__init__()
        self._type = ("OracleBaseAttachData", True)
        self._operations = (self.__undef__, True)
        self._number_of_connections = (self.__undef__, True)
        self._external_file_path = (self.__undef__, True)
        self._files_per_set = (self.__undef__, True)
        self._check_logical = (self.__undef__, True)
        self._backup_level_enabled = (self.__undef__, True)
        self._db_user = (self.__undef__, True)
        self._link_now = (self.__undef__, True)
        self._rman_channels = (self.__undef__, True)
        self._encrypted_linking_enabled = (self.__undef__, True)
        self._environment_user = (self.__undef__, True)
        self._db_credentials = (self.__undef__, True)
        self._force = (self.__undef__, True)
        self._compressed_linking_enabled = (self.__undef__, True)
        self._bandwidth_limit = (self.__undef__, True)

    API_VERSION = "1.10.0"

    @classmethod
    def from_dict(cls, data, dirty=False, undef_enabled=True):
        obj = super(OracleBaseAttachData, cls).from_dict(data, dirty, undef_enabled)
        if "operations" in data and data["operations"] is not None:
            obj._operations = (factory.create_object(data["operations"], "LinkedSourceOperations"), dirty)
            factory.validate_type(obj._operations[0], "LinkedSourceOperations")
        else:
            obj._operations = (obj.__undef__, dirty)
        obj._number_of_connections = (data.get("numberOfConnections", obj.__undef__), dirty)
        if obj._number_of_connections[0] is not None and obj._number_of_connections[0] is not obj.__undef__:
            assert isinstance(obj._number_of_connections[0], int), ("Expected one of [u'integer'], but got %s of type %s" % (obj._number_of_connections[0], type(obj._number_of_connections[0])))
            common.validate_format(obj._number_of_connections[0], "None", None, None)
        obj._external_file_path = (data.get("externalFilePath", obj.__undef__), dirty)
        if obj._external_file_path[0] is not None and obj._external_file_path[0] is not obj.__undef__:
            assert isinstance(obj._external_file_path[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._external_file_path[0], type(obj._external_file_path[0])))
            common.validate_format(obj._external_file_path[0], "None", None, 1024)
        obj._files_per_set = (data.get("filesPerSet", obj.__undef__), dirty)
        if obj._files_per_set[0] is not None and obj._files_per_set[0] is not obj.__undef__:
            assert isinstance(obj._files_per_set[0], int), ("Expected one of [u'integer'], but got %s of type %s" % (obj._files_per_set[0], type(obj._files_per_set[0])))
            common.validate_format(obj._files_per_set[0], "None", None, None)
        obj._check_logical = (data.get("checkLogical", obj.__undef__), dirty)
        if obj._check_logical[0] is not None and obj._check_logical[0] is not obj.__undef__:
            assert isinstance(obj._check_logical[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._check_logical[0], type(obj._check_logical[0])))
            common.validate_format(obj._check_logical[0], "None", None, None)
        obj._backup_level_enabled = (data.get("backupLevelEnabled", obj.__undef__), dirty)
        if obj._backup_level_enabled[0] is not None and obj._backup_level_enabled[0] is not obj.__undef__:
            assert isinstance(obj._backup_level_enabled[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._backup_level_enabled[0], type(obj._backup_level_enabled[0])))
            common.validate_format(obj._backup_level_enabled[0], "None", None, None)
        if "dbUser" not in data:
            raise ValueError("Missing required property \"dbUser\".")
        obj._db_user = (data.get("dbUser", obj.__undef__), dirty)
        if obj._db_user[0] is not None and obj._db_user[0] is not obj.__undef__:
            assert isinstance(obj._db_user[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._db_user[0], type(obj._db_user[0])))
            common.validate_format(obj._db_user[0], "None", None, None)
        obj._link_now = (data.get("linkNow", obj.__undef__), dirty)
        if obj._link_now[0] is not None and obj._link_now[0] is not obj.__undef__:
            assert isinstance(obj._link_now[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._link_now[0], type(obj._link_now[0])))
            common.validate_format(obj._link_now[0], "None", None, None)
        obj._rman_channels = (data.get("rmanChannels", obj.__undef__), dirty)
        if obj._rman_channels[0] is not None and obj._rman_channels[0] is not obj.__undef__:
            assert isinstance(obj._rman_channels[0], int), ("Expected one of [u'integer'], but got %s of type %s" % (obj._rman_channels[0], type(obj._rman_channels[0])))
            common.validate_format(obj._rman_channels[0], "None", None, None)
        obj._encrypted_linking_enabled = (data.get("encryptedLinkingEnabled", obj.__undef__), dirty)
        if obj._encrypted_linking_enabled[0] is not None and obj._encrypted_linking_enabled[0] is not obj.__undef__:
            assert isinstance(obj._encrypted_linking_enabled[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._encrypted_linking_enabled[0], type(obj._encrypted_linking_enabled[0])))
            common.validate_format(obj._encrypted_linking_enabled[0], "None", None, None)
        if "environmentUser" not in data:
            raise ValueError("Missing required property \"environmentUser\".")
        obj._environment_user = (data.get("environmentUser", obj.__undef__), dirty)
        if obj._environment_user[0] is not None and obj._environment_user[0] is not obj.__undef__:
            assert isinstance(obj._environment_user[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._environment_user[0], type(obj._environment_user[0])))
            common.validate_format(obj._environment_user[0], "objectReference", None, None)
        if "dbCredentials" not in data:
            raise ValueError("Missing required property \"dbCredentials\".")
        if "dbCredentials" in data and data["dbCredentials"] is not None:
            obj._db_credentials = (factory.create_object(data["dbCredentials"], "PasswordCredential"), dirty)
            factory.validate_type(obj._db_credentials[0], "PasswordCredential")
        else:
            obj._db_credentials = (obj.__undef__, dirty)
        obj._force = (data.get("force", obj.__undef__), dirty)
        if obj._force[0] is not None and obj._force[0] is not obj.__undef__:
            assert isinstance(obj._force[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._force[0], type(obj._force[0])))
            common.validate_format(obj._force[0], "None", None, None)
        obj._compressed_linking_enabled = (data.get("compressedLinkingEnabled", obj.__undef__), dirty)
        if obj._compressed_linking_enabled[0] is not None and obj._compressed_linking_enabled[0] is not obj.__undef__:
            assert isinstance(obj._compressed_linking_enabled[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._compressed_linking_enabled[0], type(obj._compressed_linking_enabled[0])))
            common.validate_format(obj._compressed_linking_enabled[0], "None", None, None)
        obj._bandwidth_limit = (data.get("bandwidthLimit", obj.__undef__), dirty)
        if obj._bandwidth_limit[0] is not None and obj._bandwidth_limit[0] is not obj.__undef__:
            assert isinstance(obj._bandwidth_limit[0], int), ("Expected one of [u'integer'], but got %s of type %s" % (obj._bandwidth_limit[0], type(obj._bandwidth_limit[0])))
            common.validate_format(obj._bandwidth_limit[0], "None", None, None)
        return obj

    def to_dict(self, dirty=False, belongs_to_parent=False):
        dct = super(OracleBaseAttachData, self).to_dict(dirty, belongs_to_parent)

        def dictify(obj, prop_is_list_or_vo=False):
            if isinstance(obj, list):
                return [dictify(o, prop_is_list_or_vo) for o in obj]
            elif hasattr(obj, "to_dict"):
                return obj.to_dict(dirty=dirty, belongs_to_parent=prop_is_list_or_vo)
            else:
                return obj
        if "operations" == "type" or (self.operations is not self.__undef__ and (not (dirty and not self._operations[1]) or isinstance(self.operations, list) or belongs_to_parent)):
            dct["operations"] = dictify(self.operations, prop_is_list_or_vo=True)
        if "number_of_connections" == "type" or (self.number_of_connections is not self.__undef__ and (not (dirty and not self._number_of_connections[1]) or isinstance(self.number_of_connections, list) or belongs_to_parent)):
            dct["numberOfConnections"] = dictify(self.number_of_connections)
        elif belongs_to_parent and self.number_of_connections is self.__undef__:
            dct["numberOfConnections"] = 1
        if "external_file_path" == "type" or (self.external_file_path is not self.__undef__ and (not (dirty and not self._external_file_path[1]) or isinstance(self.external_file_path, list) or belongs_to_parent)):
            dct["externalFilePath"] = dictify(self.external_file_path)
        if "files_per_set" == "type" or (self.files_per_set is not self.__undef__ and (not (dirty and not self._files_per_set[1]) or isinstance(self.files_per_set, list) or belongs_to_parent)):
            dct["filesPerSet"] = dictify(self.files_per_set)
        elif belongs_to_parent and self.files_per_set is self.__undef__:
            dct["filesPerSet"] = 5
        if "check_logical" == "type" or (self.check_logical is not self.__undef__ and (not (dirty and not self._check_logical[1]) or isinstance(self.check_logical, list) or belongs_to_parent)):
            dct["checkLogical"] = dictify(self.check_logical)
        elif belongs_to_parent and self.check_logical is self.__undef__:
            dct["checkLogical"] = False
        if "backup_level_enabled" == "type" or (self.backup_level_enabled is not self.__undef__ and (not (dirty and not self._backup_level_enabled[1]) or isinstance(self.backup_level_enabled, list) or belongs_to_parent)):
            dct["backupLevelEnabled"] = dictify(self.backup_level_enabled)
        if "db_user" == "type" or (self.db_user is not self.__undef__ and (not (dirty and not self._db_user[1]) or isinstance(self.db_user, list) or belongs_to_parent)):
            dct["dbUser"] = dictify(self.db_user)
        if "link_now" == "type" or (self.link_now is not self.__undef__ and (not (dirty and not self._link_now[1]) or isinstance(self.link_now, list) or belongs_to_parent)):
            dct["linkNow"] = dictify(self.link_now)
        elif belongs_to_parent and self.link_now is self.__undef__:
            dct["linkNow"] = False
        if "rman_channels" == "type" or (self.rman_channels is not self.__undef__ and (not (dirty and not self._rman_channels[1]) or isinstance(self.rman_channels, list) or belongs_to_parent)):
            dct["rmanChannels"] = dictify(self.rman_channels)
        elif belongs_to_parent and self.rman_channels is self.__undef__:
            dct["rmanChannels"] = 2
        if "encrypted_linking_enabled" == "type" or (self.encrypted_linking_enabled is not self.__undef__ and (not (dirty and not self._encrypted_linking_enabled[1]) or isinstance(self.encrypted_linking_enabled, list) or belongs_to_parent)):
            dct["encryptedLinkingEnabled"] = dictify(self.encrypted_linking_enabled)
        elif belongs_to_parent and self.encrypted_linking_enabled is self.__undef__:
            dct["encryptedLinkingEnabled"] = False
        if "environment_user" == "type" or (self.environment_user is not self.__undef__ and (not (dirty and not self._environment_user[1]) or isinstance(self.environment_user, list) or belongs_to_parent)):
            dct["environmentUser"] = dictify(self.environment_user)
        if "db_credentials" == "type" or (self.db_credentials is not self.__undef__ and (not (dirty and not self._db_credentials[1]) or isinstance(self.db_credentials, list) or belongs_to_parent)):
            dct["dbCredentials"] = dictify(self.db_credentials, prop_is_list_or_vo=True)
        if "force" == "type" or (self.force is not self.__undef__ and (not (dirty and not self._force[1]) or isinstance(self.force, list) or belongs_to_parent)):
            dct["force"] = dictify(self.force)
        elif belongs_to_parent and self.force is self.__undef__:
            dct["force"] = False
        if "compressed_linking_enabled" == "type" or (self.compressed_linking_enabled is not self.__undef__ and (not (dirty and not self._compressed_linking_enabled[1]) or isinstance(self.compressed_linking_enabled, list) or belongs_to_parent)):
            dct["compressedLinkingEnabled"] = dictify(self.compressed_linking_enabled)
        elif belongs_to_parent and self.compressed_linking_enabled is self.__undef__:
            dct["compressedLinkingEnabled"] = True
        if "bandwidth_limit" == "type" or (self.bandwidth_limit is not self.__undef__ and (not (dirty and not self._bandwidth_limit[1]) or isinstance(self.bandwidth_limit, list) or belongs_to_parent)):
            dct["bandwidthLimit"] = dictify(self.bandwidth_limit)
        elif belongs_to_parent and self.bandwidth_limit is self.__undef__:
            dct["bandwidthLimit"] = 0
        return dct

    def dirty(self):
        return self.from_dict(self.to_dict(dirty=False), dirty=True)

    def force_dirty(self):
        self._operations = (self._operations[0], True)
        self._number_of_connections = (self._number_of_connections[0], True)
        self._external_file_path = (self._external_file_path[0], True)
        self._files_per_set = (self._files_per_set[0], True)
        self._check_logical = (self._check_logical[0], True)
        self._backup_level_enabled = (self._backup_level_enabled[0], True)
        self._db_user = (self._db_user[0], True)
        self._link_now = (self._link_now[0], True)
        self._rman_channels = (self._rman_channels[0], True)
        self._encrypted_linking_enabled = (self._encrypted_linking_enabled[0], True)
        self._environment_user = (self._environment_user[0], True)
        self._db_credentials = (self._db_credentials[0], True)
        self._force = (self._force[0], True)
        self._compressed_linking_enabled = (self._compressed_linking_enabled[0], True)
        self._bandwidth_limit = (self._bandwidth_limit[0], True)

    def is_dirty(self):
        return any([self._operations[1], self._number_of_connections[1], self._external_file_path[1], self._files_per_set[1], self._check_logical[1], self._backup_level_enabled[1], self._db_user[1], self._link_now[1], self._rman_channels[1], self._encrypted_linking_enabled[1], self._environment_user[1], self._db_credentials[1], self._force[1], self._compressed_linking_enabled[1], self._bandwidth_limit[1]])

    def __eq__(self, other):
        if other is None:
            return False
        if not isinstance(other, OracleBaseAttachData):
            return False
        return super(OracleBaseAttachData, self).__eq__(other) and \
               self.operations == other.operations and \
               self.number_of_connections == other.number_of_connections and \
               self.external_file_path == other.external_file_path and \
               self.files_per_set == other.files_per_set and \
               self.check_logical == other.check_logical and \
               self.backup_level_enabled == other.backup_level_enabled and \
               self.db_user == other.db_user and \
               self.link_now == other.link_now and \
               self.rman_channels == other.rman_channels and \
               self.encrypted_linking_enabled == other.encrypted_linking_enabled and \
               self.environment_user == other.environment_user and \
               self.db_credentials == other.db_credentials and \
               self.force == other.force and \
               self.compressed_linking_enabled == other.compressed_linking_enabled and \
               self.bandwidth_limit == other.bandwidth_limit

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        return id(self)

    def __repr__(self):
        return common.generate_repr_string(self)

    @property
    def operations(self):
        """
        User-specified operation hooks for this source.

        :rtype: :py:class:`v1_10_0.web.vo.LinkedSourceOperations`
        """
        return self._operations[0]

    @operations.setter
    def operations(self, value):
        self._operations = (value, True)

    @property
    def number_of_connections(self):
        """
        *(default value: 1)* Total number of transport connections to use
        during SnapSync.

        :rtype: ``int``
        """
        return self._number_of_connections[0]

    @number_of_connections.setter
    def number_of_connections(self, value):
        self._number_of_connections = (value, True)

    @property
    def external_file_path(self):
        """
        External file path.

        :rtype: ``TEXT_TYPE``
        """
        return self._external_file_path[0]

    @external_file_path.setter
    def external_file_path(self, value):
        self._external_file_path = (value, True)

    @property
    def files_per_set(self):
        """
        *(default value: 5)* Number of data files to include in each RMAN
        backup set.

        :rtype: ``int``
        """
        return self._files_per_set[0]

    @files_per_set.setter
    def files_per_set(self, value):
        self._files_per_set = (value, True)

    @property
    def check_logical(self):
        """
        True if extended block checking should be used for this linked
        database.

        :rtype: ``bool``
        """
        return self._check_logical[0]

    @check_logical.setter
    def check_logical(self, value):
        self._check_logical = (value, True)

    @property
    def backup_level_enabled(self):
        """
        Defines whether backup level is enabled.

        :rtype: ``bool``
        """
        return self._backup_level_enabled[0]

    @backup_level_enabled.setter
    def backup_level_enabled(self, value):
        self._backup_level_enabled = (value, True)

    @property
    def db_user(self):
        """
        The name of the DB user.

        :rtype: ``TEXT_TYPE``
        """
        return self._db_user[0]

    @db_user.setter
    def db_user(self, value):
        self._db_user = (value, True)

    @property
    def link_now(self):
        """
        True if initial load should be done immediately.

        :rtype: ``bool``
        """
        return self._link_now[0]

    @link_now.setter
    def link_now(self, value):
        self._link_now = (value, True)

    @property
    def rman_channels(self):
        """
        *(default value: 2)* Number of parallel channels to use.

        :rtype: ``int``
        """
        return self._rman_channels[0]

    @rman_channels.setter
    def rman_channels(self, value):
        self._rman_channels = (value, True)

    @property
    def encrypted_linking_enabled(self):
        """
        True if SnapSync data from the source should be retrieved through an
        encrypted connection. Enabling this feature can decrease the
        performance of SnapSync from the source but has no impact on the
        performance of VDBs created from the retrieved data.

        :rtype: ``bool``
        """
        return self._encrypted_linking_enabled[0]

    @encrypted_linking_enabled.setter
    def encrypted_linking_enabled(self, value):
        self._encrypted_linking_enabled = (value, True)

    @property
    def environment_user(self):
        """
        Information about the OS user to use for linking.

        :rtype: ``TEXT_TYPE``
        """
        return self._environment_user[0]

    @environment_user.setter
    def environment_user(self, value):
        self._environment_user = (value, True)

    @property
    def db_credentials(self):
        """
        The password for the DB user.

        :rtype: :py:class:`v1_10_0.web.vo.PasswordCredential`
        """
        return self._db_credentials[0]

    @db_credentials.setter
    def db_credentials(self, value):
        self._db_credentials = (value, True)

    @property
    def force(self):
        """
        True if attach should succeed even if the resetlogs of the original
        database does not match the resetlogs of the new database and the
        resetlogs information of the original database is not a parent
        incarnation of the current resetlogs. This can happen when the
        controlfile has been recreated and the incarnation table is incomplete.
        Use this option with extreme caution. Attached database must be the
        same database to avoid data corruption later on.

        :rtype: ``bool``
        """
        return self._force[0]

    @force.setter
    def force(self, value):
        self._force = (value, True)

    @property
    def compressed_linking_enabled(self):
        """
        *(default value: True)* True if SnapSync data from the source should be
        compressed over the network. Enabling this feature will reduce network
        bandwidth consumption and may significantly improve throughput,
        especially over slow network.

        :rtype: ``bool``
        """
        return self._compressed_linking_enabled[0]

    @compressed_linking_enabled.setter
    def compressed_linking_enabled(self, value):
        self._compressed_linking_enabled = (value, True)

    @property
    def bandwidth_limit(self):
        """
        Bandwidth limit (MB/s) for SnapSync and LogSync network traffic. A
        value of 0 means no limit.

        :rtype: ``int``
        """
        return self._bandwidth_limit[0]

    @bandwidth_limit.setter
    def bandwidth_limit(self, value):
        self._bandwidth_limit = (value, True)




#
# Copyright 2019 by Delphix
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

#
# This class has been automatically generated from:
#     /delphix-plugin-manifest.json
#
# Do not edit this file manually!
#

from __future__ import unicode_literals

from delphixpy.v1_10_5.web.objects.TypedObject import TypedObject
from delphixpy.v1_10_5 import common

class __Undef(object):
    def __repr__(self):
        return "undef"

    def __setattr__(self, name, value):
        raise Exception('Cannot modify attributes of __Undef.')

_UNDEFINED = __Undef()

try:
    TEXT_TYPE = unicode
except NameError:
    TEXT_TYPE = str

class PluginManifest(TypedObject):
    """
    *(extends* :py:class:`v1_10_5.web.vo.TypedObject` *)* A manifest describing
    a plugin.
    """
    def __init__(self, undef_enabled=True):
        super(PluginManifest, self).__init__()
        self._type = ("PluginManifest", True)
        self._has_virtual_status = (self.__undef__, True)
        self._has_linked_status = (self.__undef__, True)
        self._has_linked_pre_snapshot = (self.__undef__, True)
        self._has_virtual_stop = (self.__undef__, True)
        self._has_virtual_reconfigure = (self.__undef__, True)
        self._has_virtual_configure = (self.__undef__, True)
        self._has_linked_worker = (self.__undef__, True)
        self._has_source_config_discovery = (self.__undef__, True)
        self._has_virtual_start = (self.__undef__, True)
        self._has_linked_stop_staging = (self.__undef__, True)
        self._has_repository_discovery = (self.__undef__, True)
        self._has_linked_start_staging = (self.__undef__, True)
        self._has_linked_mount_specification = (self.__undef__, True)
        self._has_initialize = (self.__undef__, True)
        self._has_virtual_post_snapshot = (self.__undef__, True)
        self._has_virtual_mount_specification = (self.__undef__, True)
        self._has_linked_post_snapshot = (self.__undef__, True)
        self._has_virtual_pre_snapshot = (self.__undef__, True)
        self._has_virtual_unconfigure = (self.__undef__, True)

    API_VERSION = "1.10.5"

    @classmethod
    def from_dict(cls, data, dirty=False, undef_enabled=True):
        obj = super(PluginManifest, cls).from_dict(data, dirty, undef_enabled)
        if "hasVirtualStatus" not in data:
            raise ValueError("Missing required property \"hasVirtualStatus\".")
        obj._has_virtual_status = (data.get("hasVirtualStatus", obj.__undef__), dirty)
        if obj._has_virtual_status[0] is not None and obj._has_virtual_status[0] is not obj.__undef__:
            assert isinstance(obj._has_virtual_status[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._has_virtual_status[0], type(obj._has_virtual_status[0])))
            common.validate_format(obj._has_virtual_status[0], "None", None, None)
        if "hasLinkedStatus" not in data:
            raise ValueError("Missing required property \"hasLinkedStatus\".")
        obj._has_linked_status = (data.get("hasLinkedStatus", obj.__undef__), dirty)
        if obj._has_linked_status[0] is not None and obj._has_linked_status[0] is not obj.__undef__:
            assert isinstance(obj._has_linked_status[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._has_linked_status[0], type(obj._has_linked_status[0])))
            common.validate_format(obj._has_linked_status[0], "None", None, None)
        if "hasLinkedPreSnapshot" not in data:
            raise ValueError("Missing required property \"hasLinkedPreSnapshot\".")
        obj._has_linked_pre_snapshot = (data.get("hasLinkedPreSnapshot", obj.__undef__), dirty)
        if obj._has_linked_pre_snapshot[0] is not None and obj._has_linked_pre_snapshot[0] is not obj.__undef__:
            assert isinstance(obj._has_linked_pre_snapshot[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._has_linked_pre_snapshot[0], type(obj._has_linked_pre_snapshot[0])))
            common.validate_format(obj._has_linked_pre_snapshot[0], "None", None, None)
        if "hasVirtualStop" not in data:
            raise ValueError("Missing required property \"hasVirtualStop\".")
        obj._has_virtual_stop = (data.get("hasVirtualStop", obj.__undef__), dirty)
        if obj._has_virtual_stop[0] is not None and obj._has_virtual_stop[0] is not obj.__undef__:
            assert isinstance(obj._has_virtual_stop[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._has_virtual_stop[0], type(obj._has_virtual_stop[0])))
            common.validate_format(obj._has_virtual_stop[0], "None", None, None)
        if "hasVirtualReconfigure" not in data:
            raise ValueError("Missing required property \"hasVirtualReconfigure\".")
        obj._has_virtual_reconfigure = (data.get("hasVirtualReconfigure", obj.__undef__), dirty)
        if obj._has_virtual_reconfigure[0] is not None and obj._has_virtual_reconfigure[0] is not obj.__undef__:
            assert isinstance(obj._has_virtual_reconfigure[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._has_virtual_reconfigure[0], type(obj._has_virtual_reconfigure[0])))
            common.validate_format(obj._has_virtual_reconfigure[0], "None", None, None)
        if "hasVirtualConfigure" not in data:
            raise ValueError("Missing required property \"hasVirtualConfigure\".")
        obj._has_virtual_configure = (data.get("hasVirtualConfigure", obj.__undef__), dirty)
        if obj._has_virtual_configure[0] is not None and obj._has_virtual_configure[0] is not obj.__undef__:
            assert isinstance(obj._has_virtual_configure[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._has_virtual_configure[0], type(obj._has_virtual_configure[0])))
            common.validate_format(obj._has_virtual_configure[0], "None", None, None)
        if "hasLinkedWorker" not in data:
            raise ValueError("Missing required property \"hasLinkedWorker\".")
        obj._has_linked_worker = (data.get("hasLinkedWorker", obj.__undef__), dirty)
        if obj._has_linked_worker[0] is not None and obj._has_linked_worker[0] is not obj.__undef__:
            assert isinstance(obj._has_linked_worker[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._has_linked_worker[0], type(obj._has_linked_worker[0])))
            common.validate_format(obj._has_linked_worker[0], "None", None, None)
        if "hasSourceConfigDiscovery" not in data:
            raise ValueError("Missing required property \"hasSourceConfigDiscovery\".")
        obj._has_source_config_discovery = (data.get("hasSourceConfigDiscovery", obj.__undef__), dirty)
        if obj._has_source_config_discovery[0] is not None and obj._has_source_config_discovery[0] is not obj.__undef__:
            assert isinstance(obj._has_source_config_discovery[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._has_source_config_discovery[0], type(obj._has_source_config_discovery[0])))
            common.validate_format(obj._has_source_config_discovery[0], "None", None, None)
        if "hasVirtualStart" not in data:
            raise ValueError("Missing required property \"hasVirtualStart\".")
        obj._has_virtual_start = (data.get("hasVirtualStart", obj.__undef__), dirty)
        if obj._has_virtual_start[0] is not None and obj._has_virtual_start[0] is not obj.__undef__:
            assert isinstance(obj._has_virtual_start[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._has_virtual_start[0], type(obj._has_virtual_start[0])))
            common.validate_format(obj._has_virtual_start[0], "None", None, None)
        if "hasLinkedStopStaging" not in data:
            raise ValueError("Missing required property \"hasLinkedStopStaging\".")
        obj._has_linked_stop_staging = (data.get("hasLinkedStopStaging", obj.__undef__), dirty)
        if obj._has_linked_stop_staging[0] is not None and obj._has_linked_stop_staging[0] is not obj.__undef__:
            assert isinstance(obj._has_linked_stop_staging[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._has_linked_stop_staging[0], type(obj._has_linked_stop_staging[0])))
            common.validate_format(obj._has_linked_stop_staging[0], "None", None, None)
        if "hasRepositoryDiscovery" not in data:
            raise ValueError("Missing required property \"hasRepositoryDiscovery\".")
        obj._has_repository_discovery = (data.get("hasRepositoryDiscovery", obj.__undef__), dirty)
        if obj._has_repository_discovery[0] is not None and obj._has_repository_discovery[0] is not obj.__undef__:
            assert isinstance(obj._has_repository_discovery[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._has_repository_discovery[0], type(obj._has_repository_discovery[0])))
            common.validate_format(obj._has_repository_discovery[0], "None", None, None)
        if "hasLinkedStartStaging" not in data:
            raise ValueError("Missing required property \"hasLinkedStartStaging\".")
        obj._has_linked_start_staging = (data.get("hasLinkedStartStaging", obj.__undef__), dirty)
        if obj._has_linked_start_staging[0] is not None and obj._has_linked_start_staging[0] is not obj.__undef__:
            assert isinstance(obj._has_linked_start_staging[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._has_linked_start_staging[0], type(obj._has_linked_start_staging[0])))
            common.validate_format(obj._has_linked_start_staging[0], "None", None, None)
        if "hasLinkedMountSpecification" not in data:
            raise ValueError("Missing required property \"hasLinkedMountSpecification\".")
        obj._has_linked_mount_specification = (data.get("hasLinkedMountSpecification", obj.__undef__), dirty)
        if obj._has_linked_mount_specification[0] is not None and obj._has_linked_mount_specification[0] is not obj.__undef__:
            assert isinstance(obj._has_linked_mount_specification[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._has_linked_mount_specification[0], type(obj._has_linked_mount_specification[0])))
            common.validate_format(obj._has_linked_mount_specification[0], "None", None, None)
        if "hasInitialize" not in data:
            raise ValueError("Missing required property \"hasInitialize\".")
        obj._has_initialize = (data.get("hasInitialize", obj.__undef__), dirty)
        if obj._has_initialize[0] is not None and obj._has_initialize[0] is not obj.__undef__:
            assert isinstance(obj._has_initialize[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._has_initialize[0], type(obj._has_initialize[0])))
            common.validate_format(obj._has_initialize[0], "None", None, None)
        if "hasVirtualPostSnapshot" not in data:
            raise ValueError("Missing required property \"hasVirtualPostSnapshot\".")
        obj._has_virtual_post_snapshot = (data.get("hasVirtualPostSnapshot", obj.__undef__), dirty)
        if obj._has_virtual_post_snapshot[0] is not None and obj._has_virtual_post_snapshot[0] is not obj.__undef__:
            assert isinstance(obj._has_virtual_post_snapshot[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._has_virtual_post_snapshot[0], type(obj._has_virtual_post_snapshot[0])))
            common.validate_format(obj._has_virtual_post_snapshot[0], "None", None, None)
        if "hasVirtualMountSpecification" not in data:
            raise ValueError("Missing required property \"hasVirtualMountSpecification\".")
        obj._has_virtual_mount_specification = (data.get("hasVirtualMountSpecification", obj.__undef__), dirty)
        if obj._has_virtual_mount_specification[0] is not None and obj._has_virtual_mount_specification[0] is not obj.__undef__:
            assert isinstance(obj._has_virtual_mount_specification[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._has_virtual_mount_specification[0], type(obj._has_virtual_mount_specification[0])))
            common.validate_format(obj._has_virtual_mount_specification[0], "None", None, None)
        if "hasLinkedPostSnapshot" not in data:
            raise ValueError("Missing required property \"hasLinkedPostSnapshot\".")
        obj._has_linked_post_snapshot = (data.get("hasLinkedPostSnapshot", obj.__undef__), dirty)
        if obj._has_linked_post_snapshot[0] is not None and obj._has_linked_post_snapshot[0] is not obj.__undef__:
            assert isinstance(obj._has_linked_post_snapshot[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._has_linked_post_snapshot[0], type(obj._has_linked_post_snapshot[0])))
            common.validate_format(obj._has_linked_post_snapshot[0], "None", None, None)
        if "hasVirtualPreSnapshot" not in data:
            raise ValueError("Missing required property \"hasVirtualPreSnapshot\".")
        obj._has_virtual_pre_snapshot = (data.get("hasVirtualPreSnapshot", obj.__undef__), dirty)
        if obj._has_virtual_pre_snapshot[0] is not None and obj._has_virtual_pre_snapshot[0] is not obj.__undef__:
            assert isinstance(obj._has_virtual_pre_snapshot[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._has_virtual_pre_snapshot[0], type(obj._has_virtual_pre_snapshot[0])))
            common.validate_format(obj._has_virtual_pre_snapshot[0], "None", None, None)
        if "hasVirtualUnconfigure" not in data:
            raise ValueError("Missing required property \"hasVirtualUnconfigure\".")
        obj._has_virtual_unconfigure = (data.get("hasVirtualUnconfigure", obj.__undef__), dirty)
        if obj._has_virtual_unconfigure[0] is not None and obj._has_virtual_unconfigure[0] is not obj.__undef__:
            assert isinstance(obj._has_virtual_unconfigure[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._has_virtual_unconfigure[0], type(obj._has_virtual_unconfigure[0])))
            common.validate_format(obj._has_virtual_unconfigure[0], "None", None, None)
        return obj

    def to_dict(self, dirty=False, belongs_to_parent=False):
        dct = super(PluginManifest, self).to_dict(dirty, belongs_to_parent)

        def dictify(obj, prop_is_list_or_vo=False):
            if isinstance(obj, list):
                return [dictify(o, prop_is_list_or_vo) for o in obj]
            elif hasattr(obj, "to_dict"):
                return obj.to_dict(dirty=dirty, belongs_to_parent=prop_is_list_or_vo)
            else:
                return obj
        if "has_virtual_status" == "type" or (self.has_virtual_status is not self.__undef__ and (not (dirty and not self._has_virtual_status[1]) or isinstance(self.has_virtual_status, list) or belongs_to_parent)):
            dct["hasVirtualStatus"] = dictify(self.has_virtual_status)
        if "has_linked_status" == "type" or (self.has_linked_status is not self.__undef__ and (not (dirty and not self._has_linked_status[1]) or isinstance(self.has_linked_status, list) or belongs_to_parent)):
            dct["hasLinkedStatus"] = dictify(self.has_linked_status)
        if "has_linked_pre_snapshot" == "type" or (self.has_linked_pre_snapshot is not self.__undef__ and (not (dirty and not self._has_linked_pre_snapshot[1]) or isinstance(self.has_linked_pre_snapshot, list) or belongs_to_parent)):
            dct["hasLinkedPreSnapshot"] = dictify(self.has_linked_pre_snapshot)
        if "has_virtual_stop" == "type" or (self.has_virtual_stop is not self.__undef__ and (not (dirty and not self._has_virtual_stop[1]) or isinstance(self.has_virtual_stop, list) or belongs_to_parent)):
            dct["hasVirtualStop"] = dictify(self.has_virtual_stop)
        if "has_virtual_reconfigure" == "type" or (self.has_virtual_reconfigure is not self.__undef__ and (not (dirty and not self._has_virtual_reconfigure[1]) or isinstance(self.has_virtual_reconfigure, list) or belongs_to_parent)):
            dct["hasVirtualReconfigure"] = dictify(self.has_virtual_reconfigure)
        if "has_virtual_configure" == "type" or (self.has_virtual_configure is not self.__undef__ and (not (dirty and not self._has_virtual_configure[1]) or isinstance(self.has_virtual_configure, list) or belongs_to_parent)):
            dct["hasVirtualConfigure"] = dictify(self.has_virtual_configure)
        if "has_linked_worker" == "type" or (self.has_linked_worker is not self.__undef__ and (not (dirty and not self._has_linked_worker[1]) or isinstance(self.has_linked_worker, list) or belongs_to_parent)):
            dct["hasLinkedWorker"] = dictify(self.has_linked_worker)
        if "has_source_config_discovery" == "type" or (self.has_source_config_discovery is not self.__undef__ and (not (dirty and not self._has_source_config_discovery[1]) or isinstance(self.has_source_config_discovery, list) or belongs_to_parent)):
            dct["hasSourceConfigDiscovery"] = dictify(self.has_source_config_discovery)
        if "has_virtual_start" == "type" or (self.has_virtual_start is not self.__undef__ and (not (dirty and not self._has_virtual_start[1]) or isinstance(self.has_virtual_start, list) or belongs_to_parent)):
            dct["hasVirtualStart"] = dictify(self.has_virtual_start)
        if "has_linked_stop_staging" == "type" or (self.has_linked_stop_staging is not self.__undef__ and (not (dirty and not self._has_linked_stop_staging[1]) or isinstance(self.has_linked_stop_staging, list) or belongs_to_parent)):
            dct["hasLinkedStopStaging"] = dictify(self.has_linked_stop_staging)
        if "has_repository_discovery" == "type" or (self.has_repository_discovery is not self.__undef__ and (not (dirty and not self._has_repository_discovery[1]) or isinstance(self.has_repository_discovery, list) or belongs_to_parent)):
            dct["hasRepositoryDiscovery"] = dictify(self.has_repository_discovery)
        if "has_linked_start_staging" == "type" or (self.has_linked_start_staging is not self.__undef__ and (not (dirty and not self._has_linked_start_staging[1]) or isinstance(self.has_linked_start_staging, list) or belongs_to_parent)):
            dct["hasLinkedStartStaging"] = dictify(self.has_linked_start_staging)
        if "has_linked_mount_specification" == "type" or (self.has_linked_mount_specification is not self.__undef__ and (not (dirty and not self._has_linked_mount_specification[1]) or isinstance(self.has_linked_mount_specification, list) or belongs_to_parent)):
            dct["hasLinkedMountSpecification"] = dictify(self.has_linked_mount_specification)
        if "has_initialize" == "type" or (self.has_initialize is not self.__undef__ and (not (dirty and not self._has_initialize[1]) or isinstance(self.has_initialize, list) or belongs_to_parent)):
            dct["hasInitialize"] = dictify(self.has_initialize)
        if "has_virtual_post_snapshot" == "type" or (self.has_virtual_post_snapshot is not self.__undef__ and (not (dirty and not self._has_virtual_post_snapshot[1]) or isinstance(self.has_virtual_post_snapshot, list) or belongs_to_parent)):
            dct["hasVirtualPostSnapshot"] = dictify(self.has_virtual_post_snapshot)
        if "has_virtual_mount_specification" == "type" or (self.has_virtual_mount_specification is not self.__undef__ and (not (dirty and not self._has_virtual_mount_specification[1]) or isinstance(self.has_virtual_mount_specification, list) or belongs_to_parent)):
            dct["hasVirtualMountSpecification"] = dictify(self.has_virtual_mount_specification)
        if "has_linked_post_snapshot" == "type" or (self.has_linked_post_snapshot is not self.__undef__ and (not (dirty and not self._has_linked_post_snapshot[1]) or isinstance(self.has_linked_post_snapshot, list) or belongs_to_parent)):
            dct["hasLinkedPostSnapshot"] = dictify(self.has_linked_post_snapshot)
        if "has_virtual_pre_snapshot" == "type" or (self.has_virtual_pre_snapshot is not self.__undef__ and (not (dirty and not self._has_virtual_pre_snapshot[1]) or isinstance(self.has_virtual_pre_snapshot, list) or belongs_to_parent)):
            dct["hasVirtualPreSnapshot"] = dictify(self.has_virtual_pre_snapshot)
        if "has_virtual_unconfigure" == "type" or (self.has_virtual_unconfigure is not self.__undef__ and (not (dirty and not self._has_virtual_unconfigure[1]) or isinstance(self.has_virtual_unconfigure, list) or belongs_to_parent)):
            dct["hasVirtualUnconfigure"] = dictify(self.has_virtual_unconfigure)
        return dct

    def dirty(self):
        return self.from_dict(self.to_dict(dirty=False), dirty=True)

    def force_dirty(self):
        self._has_virtual_status = (self._has_virtual_status[0], True)
        self._has_linked_status = (self._has_linked_status[0], True)
        self._has_linked_pre_snapshot = (self._has_linked_pre_snapshot[0], True)
        self._has_virtual_stop = (self._has_virtual_stop[0], True)
        self._has_virtual_reconfigure = (self._has_virtual_reconfigure[0], True)
        self._has_virtual_configure = (self._has_virtual_configure[0], True)
        self._has_linked_worker = (self._has_linked_worker[0], True)
        self._has_source_config_discovery = (self._has_source_config_discovery[0], True)
        self._has_virtual_start = (self._has_virtual_start[0], True)
        self._has_linked_stop_staging = (self._has_linked_stop_staging[0], True)
        self._has_repository_discovery = (self._has_repository_discovery[0], True)
        self._has_linked_start_staging = (self._has_linked_start_staging[0], True)
        self._has_linked_mount_specification = (self._has_linked_mount_specification[0], True)
        self._has_initialize = (self._has_initialize[0], True)
        self._has_virtual_post_snapshot = (self._has_virtual_post_snapshot[0], True)
        self._has_virtual_mount_specification = (self._has_virtual_mount_specification[0], True)
        self._has_linked_post_snapshot = (self._has_linked_post_snapshot[0], True)
        self._has_virtual_pre_snapshot = (self._has_virtual_pre_snapshot[0], True)
        self._has_virtual_unconfigure = (self._has_virtual_unconfigure[0], True)

    def is_dirty(self):
        return any([self._has_virtual_status[1], self._has_linked_status[1], self._has_linked_pre_snapshot[1], self._has_virtual_stop[1], self._has_virtual_reconfigure[1], self._has_virtual_configure[1], self._has_linked_worker[1], self._has_source_config_discovery[1], self._has_virtual_start[1], self._has_linked_stop_staging[1], self._has_repository_discovery[1], self._has_linked_start_staging[1], self._has_linked_mount_specification[1], self._has_initialize[1], self._has_virtual_post_snapshot[1], self._has_virtual_mount_specification[1], self._has_linked_post_snapshot[1], self._has_virtual_pre_snapshot[1], self._has_virtual_unconfigure[1]])

    def __eq__(self, other):
        if other is None:
            return False
        if not isinstance(other, PluginManifest):
            return False
        return super(PluginManifest, self).__eq__(other) and \
               self.has_virtual_status == other.has_virtual_status and \
               self.has_linked_status == other.has_linked_status and \
               self.has_linked_pre_snapshot == other.has_linked_pre_snapshot and \
               self.has_virtual_stop == other.has_virtual_stop and \
               self.has_virtual_reconfigure == other.has_virtual_reconfigure and \
               self.has_virtual_configure == other.has_virtual_configure and \
               self.has_linked_worker == other.has_linked_worker and \
               self.has_source_config_discovery == other.has_source_config_discovery and \
               self.has_virtual_start == other.has_virtual_start and \
               self.has_linked_stop_staging == other.has_linked_stop_staging and \
               self.has_repository_discovery == other.has_repository_discovery and \
               self.has_linked_start_staging == other.has_linked_start_staging and \
               self.has_linked_mount_specification == other.has_linked_mount_specification and \
               self.has_initialize == other.has_initialize and \
               self.has_virtual_post_snapshot == other.has_virtual_post_snapshot and \
               self.has_virtual_mount_specification == other.has_virtual_mount_specification and \
               self.has_linked_post_snapshot == other.has_linked_post_snapshot and \
               self.has_virtual_pre_snapshot == other.has_virtual_pre_snapshot and \
               self.has_virtual_unconfigure == other.has_virtual_unconfigure

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        return id(self)

    def __repr__(self):
        return common.generate_repr_string(self)

    @property
    def has_virtual_status(self):
        """
        Indicates whether virtual.status() operation has been implemented.

        :rtype: ``bool``
        """
        return self._has_virtual_status[0]

    @has_virtual_status.setter
    def has_virtual_status(self, value):
        self._has_virtual_status = (value, True)

    @property
    def has_linked_status(self):
        """
        Indicates whether linked.status() operation has been implemented.

        :rtype: ``bool``
        """
        return self._has_linked_status[0]

    @has_linked_status.setter
    def has_linked_status(self, value):
        self._has_linked_status = (value, True)

    @property
    def has_linked_pre_snapshot(self):
        """
        Indicates whether linked.pre_snapshot() operation has been implemented.

        :rtype: ``bool``
        """
        return self._has_linked_pre_snapshot[0]

    @has_linked_pre_snapshot.setter
    def has_linked_pre_snapshot(self, value):
        self._has_linked_pre_snapshot = (value, True)

    @property
    def has_virtual_stop(self):
        """
        Indicates whether virtual.stop() operation has been implemented.

        :rtype: ``bool``
        """
        return self._has_virtual_stop[0]

    @has_virtual_stop.setter
    def has_virtual_stop(self, value):
        self._has_virtual_stop = (value, True)

    @property
    def has_virtual_reconfigure(self):
        """
        Indicates whether virtual.reconfigure() operation has been implemented.

        :rtype: ``bool``
        """
        return self._has_virtual_reconfigure[0]

    @has_virtual_reconfigure.setter
    def has_virtual_reconfigure(self, value):
        self._has_virtual_reconfigure = (value, True)

    @property
    def has_virtual_configure(self):
        """
        Indicates whether virtual.configure() operation has been implemented.

        :rtype: ``bool``
        """
        return self._has_virtual_configure[0]

    @has_virtual_configure.setter
    def has_virtual_configure(self, value):
        self._has_virtual_configure = (value, True)

    @property
    def has_linked_worker(self):
        """
        Indicates whether linked.worker() operation has been implemented.

        :rtype: ``bool``
        """
        return self._has_linked_worker[0]

    @has_linked_worker.setter
    def has_linked_worker(self, value):
        self._has_linked_worker = (value, True)

    @property
    def has_source_config_discovery(self):
        """
        Indicates whether discovery.source_config() operation has been
        implemented.

        :rtype: ``bool``
        """
        return self._has_source_config_discovery[0]

    @has_source_config_discovery.setter
    def has_source_config_discovery(self, value):
        self._has_source_config_discovery = (value, True)

    @property
    def has_virtual_start(self):
        """
        Indicates whether virtual.start() operation has been implemented.

        :rtype: ``bool``
        """
        return self._has_virtual_start[0]

    @has_virtual_start.setter
    def has_virtual_start(self, value):
        self._has_virtual_start = (value, True)

    @property
    def has_linked_stop_staging(self):
        """
        Indicates whether linked.stop_staging() operation has been implemented.

        :rtype: ``bool``
        """
        return self._has_linked_stop_staging[0]

    @has_linked_stop_staging.setter
    def has_linked_stop_staging(self, value):
        self._has_linked_stop_staging = (value, True)

    @property
    def has_repository_discovery(self):
        """
        Indicates whether discovery.repository() operation has been
        implemented.

        :rtype: ``bool``
        """
        return self._has_repository_discovery[0]

    @has_repository_discovery.setter
    def has_repository_discovery(self, value):
        self._has_repository_discovery = (value, True)

    @property
    def has_linked_start_staging(self):
        """
        Indicates whether linked.start_staging() operation has been
        implemented.

        :rtype: ``bool``
        """
        return self._has_linked_start_staging[0]

    @has_linked_start_staging.setter
    def has_linked_start_staging(self, value):
        self._has_linked_start_staging = (value, True)

    @property
    def has_linked_mount_specification(self):
        """
        Indicates whether linked.mount_specification() operation has been
        implemented.

        :rtype: ``bool``
        """
        return self._has_linked_mount_specification[0]

    @has_linked_mount_specification.setter
    def has_linked_mount_specification(self, value):
        self._has_linked_mount_specification = (value, True)

    @property
    def has_initialize(self):
        """
        Indicates whether virtual.initialize() operation has been implemented.

        :rtype: ``bool``
        """
        return self._has_initialize[0]

    @has_initialize.setter
    def has_initialize(self, value):
        self._has_initialize = (value, True)

    @property
    def has_virtual_post_snapshot(self):
        """
        Indicates whether virtual.post_snapshot() operation has been
        implemented.

        :rtype: ``bool``
        """
        return self._has_virtual_post_snapshot[0]

    @has_virtual_post_snapshot.setter
    def has_virtual_post_snapshot(self, value):
        self._has_virtual_post_snapshot = (value, True)

    @property
    def has_virtual_mount_specification(self):
        """
        Indicates whether virtual.mount_specification() operation has been
        implemented.

        :rtype: ``bool``
        """
        return self._has_virtual_mount_specification[0]

    @has_virtual_mount_specification.setter
    def has_virtual_mount_specification(self, value):
        self._has_virtual_mount_specification = (value, True)

    @property
    def has_linked_post_snapshot(self):
        """
        Indicates whether linked.post_snapshot() operation has been
        implemented.

        :rtype: ``bool``
        """
        return self._has_linked_post_snapshot[0]

    @has_linked_post_snapshot.setter
    def has_linked_post_snapshot(self, value):
        self._has_linked_post_snapshot = (value, True)

    @property
    def has_virtual_pre_snapshot(self):
        """
        Indicates whether virtual.pre_snapshot() operation has been
        implemented.

        :rtype: ``bool``
        """
        return self._has_virtual_pre_snapshot[0]

    @has_virtual_pre_snapshot.setter
    def has_virtual_pre_snapshot(self, value):
        self._has_virtual_pre_snapshot = (value, True)

    @property
    def has_virtual_unconfigure(self):
        """
        Indicates whether virtual.unconfigure() operation has been implemented.

        :rtype: ``bool``
        """
        return self._has_virtual_unconfigure[0]

    @has_virtual_unconfigure.setter
    def has_virtual_unconfigure(self, value):
        self._has_virtual_unconfigure = (value, True)




#
# Copyright 2019 by Delphix
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

#
# This class has been automatically generated from:
#     /delphix-about.json
#
# Do not edit this file manually!
#

from __future__ import unicode_literals

from delphixpy.v1_10_2.web.objects.TypedObject import TypedObject
from delphixpy.v1_10_2 import factory
from delphixpy.v1_10_2 import common

class __Undef(object):
    def __repr__(self):
        return "undef"

    def __setattr__(self, name, value):
        raise Exception('Cannot modify attributes of __Undef.')

_UNDEFINED = __Undef()

try:
    TEXT_TYPE = unicode
except NameError:
    TEXT_TYPE = str

class PublicSystemInfo(TypedObject):
    """
    *(extends* :py:class:`v1_10_2.web.vo.TypedObject` *)* Retrieve static
    system-wide properties.
    """
    def __init__(self, undef_enabled=True):
        super(PublicSystemInfo, self).__init__()
        self._type = ("PublicSystemInfo", True)
        self._vendor_phone_number = (self.__undef__, True)
        self._current_locale = (self.__undef__, True)
        self._vendor_url = (self.__undef__, True)
        self._enabled_features = (self.__undef__, True)
        self._vendor_address = (self.__undef__, True)
        self._build_version = (self.__undef__, True)
        self._engine_type = (self.__undef__, True)
        self._support_url = (self.__undef__, True)
        self._support_contacts = (self.__undef__, True)
        self._api_version = (self.__undef__, True)
        self._vendor_email = (self.__undef__, True)
        self._engine_qualifier = (self.__undef__, True)
        self._build_timestamp = (self.__undef__, True)
        self._product_type = (self.__undef__, True)
        self._build_title = (self.__undef__, True)
        self._vendor_name = (self.__undef__, True)
        self._kernel_name = (self.__undef__, True)
        self._product_name = (self.__undef__, True)
        self._banner = (self.__undef__, True)
        self._configured = (self.__undef__, True)
        self._locales = (self.__undef__, True)

    API_VERSION = "1.10.2"

    @classmethod
    def from_dict(cls, data, dirty=False, undef_enabled=True):
        obj = super(PublicSystemInfo, cls).from_dict(data, dirty, undef_enabled)
        obj._vendor_phone_number = (data.get("vendorPhoneNumber", obj.__undef__), dirty)
        if obj._vendor_phone_number[0] is not None and obj._vendor_phone_number[0] is not obj.__undef__:
            assert isinstance(obj._vendor_phone_number[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._vendor_phone_number[0], type(obj._vendor_phone_number[0])))
            common.validate_format(obj._vendor_phone_number[0], "None", None, None)
        obj._current_locale = (data.get("currentLocale", obj.__undef__), dirty)
        if obj._current_locale[0] is not None and obj._current_locale[0] is not obj.__undef__:
            assert isinstance(obj._current_locale[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._current_locale[0], type(obj._current_locale[0])))
            common.validate_format(obj._current_locale[0], "locale", None, None)
        obj._vendor_url = (data.get("vendorURL", obj.__undef__), dirty)
        if obj._vendor_url[0] is not None and obj._vendor_url[0] is not obj.__undef__:
            assert isinstance(obj._vendor_url[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._vendor_url[0], type(obj._vendor_url[0])))
            common.validate_format(obj._vendor_url[0], "None", None, None)
        obj._enabled_features = []
        for item in data.get("enabledFeatures") or []:
            assert isinstance(item, TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (item, type(item)))
            common.validate_format(item, "None", None, None)
            obj._enabled_features.append(item)
        obj._enabled_features = (obj._enabled_features, dirty)
        obj._vendor_address = []
        for item in data.get("vendorAddress") or []:
            assert isinstance(item, TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (item, type(item)))
            common.validate_format(item, "None", None, None)
            obj._vendor_address.append(item)
        obj._vendor_address = (obj._vendor_address, dirty)
        if "buildVersion" in data and data["buildVersion"] is not None:
            obj._build_version = (factory.create_object(data["buildVersion"], "VersionInfo"), dirty)
            factory.validate_type(obj._build_version[0], "VersionInfo")
        else:
            obj._build_version = (obj.__undef__, dirty)
        obj._engine_type = (data.get("engineType", obj.__undef__), dirty)
        if obj._engine_type[0] is not None and obj._engine_type[0] is not obj.__undef__:
            assert isinstance(obj._engine_type[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._engine_type[0], type(obj._engine_type[0])))
            assert obj._engine_type[0] in [u'VIRTUALIZATION', u'MASKING', u'BOTH', u'UNSET'], "Expected enum [u'VIRTUALIZATION', u'MASKING', u'BOTH', u'UNSET'] but got %s" % obj._engine_type[0]
            common.validate_format(obj._engine_type[0], "None", None, None)
        obj._support_url = (data.get("supportURL", obj.__undef__), dirty)
        if obj._support_url[0] is not None and obj._support_url[0] is not obj.__undef__:
            assert isinstance(obj._support_url[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._support_url[0], type(obj._support_url[0])))
            common.validate_format(obj._support_url[0], "None", None, None)
        obj._support_contacts = []
        for item in data.get("supportContacts") or []:
            obj._support_contacts.append(factory.create_object(item))
            factory.validate_type(obj._support_contacts[-1], "SupportContact")
        obj._support_contacts = (obj._support_contacts, dirty)
        if "apiVersion" in data and data["apiVersion"] is not None:
            obj._api_version = (factory.create_object(data["apiVersion"], "APIVersion"), dirty)
            factory.validate_type(obj._api_version[0], "APIVersion")
        else:
            obj._api_version = (obj.__undef__, dirty)
        obj._vendor_email = (data.get("vendorEmail", obj.__undef__), dirty)
        if obj._vendor_email[0] is not None and obj._vendor_email[0] is not obj.__undef__:
            assert isinstance(obj._vendor_email[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._vendor_email[0], type(obj._vendor_email[0])))
            common.validate_format(obj._vendor_email[0], "email", None, None)
        obj._engine_qualifier = (data.get("engineQualifier", obj.__undef__), dirty)
        if obj._engine_qualifier[0] is not None and obj._engine_qualifier[0] is not obj.__undef__:
            assert isinstance(obj._engine_qualifier[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._engine_qualifier[0], type(obj._engine_qualifier[0])))
            common.validate_format(obj._engine_qualifier[0], "None", None, None)
        obj._build_timestamp = (data.get("buildTimestamp", obj.__undef__), dirty)
        if obj._build_timestamp[0] is not None and obj._build_timestamp[0] is not obj.__undef__:
            assert isinstance(obj._build_timestamp[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._build_timestamp[0], type(obj._build_timestamp[0])))
            common.validate_format(obj._build_timestamp[0], "date", None, None)
        obj._product_type = (data.get("productType", obj.__undef__), dirty)
        if obj._product_type[0] is not None and obj._product_type[0] is not obj.__undef__:
            assert isinstance(obj._product_type[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._product_type[0], type(obj._product_type[0])))
            common.validate_format(obj._product_type[0], "None", None, None)
        obj._build_title = (data.get("buildTitle", obj.__undef__), dirty)
        if obj._build_title[0] is not None and obj._build_title[0] is not obj.__undef__:
            assert isinstance(obj._build_title[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._build_title[0], type(obj._build_title[0])))
            common.validate_format(obj._build_title[0], "None", None, None)
        obj._vendor_name = (data.get("vendorName", obj.__undef__), dirty)
        if obj._vendor_name[0] is not None and obj._vendor_name[0] is not obj.__undef__:
            assert isinstance(obj._vendor_name[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._vendor_name[0], type(obj._vendor_name[0])))
            common.validate_format(obj._vendor_name[0], "None", None, None)
        obj._kernel_name = (data.get("kernelName", obj.__undef__), dirty)
        if obj._kernel_name[0] is not None and obj._kernel_name[0] is not obj.__undef__:
            assert isinstance(obj._kernel_name[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._kernel_name[0], type(obj._kernel_name[0])))
            common.validate_format(obj._kernel_name[0], "None", None, None)
        obj._product_name = (data.get("productName", obj.__undef__), dirty)
        if obj._product_name[0] is not None and obj._product_name[0] is not obj.__undef__:
            assert isinstance(obj._product_name[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._product_name[0], type(obj._product_name[0])))
            common.validate_format(obj._product_name[0], "None", None, None)
        obj._banner = (data.get("banner", obj.__undef__), dirty)
        if obj._banner[0] is not None and obj._banner[0] is not obj.__undef__:
            assert isinstance(obj._banner[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._banner[0], type(obj._banner[0])))
            common.validate_format(obj._banner[0], "None", None, None)
        obj._configured = (data.get("configured", obj.__undef__), dirty)
        if obj._configured[0] is not None and obj._configured[0] is not obj.__undef__:
            assert isinstance(obj._configured[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._configured[0], type(obj._configured[0])))
            common.validate_format(obj._configured[0], "None", None, None)
        obj._locales = []
        for item in data.get("locales") or []:
            assert isinstance(item, TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (item, type(item)))
            common.validate_format(item, "locale", None, None)
            obj._locales.append(item)
        obj._locales = (obj._locales, dirty)
        return obj

    def to_dict(self, dirty=False, belongs_to_parent=False):
        dct = super(PublicSystemInfo, self).to_dict(dirty, belongs_to_parent)

        def dictify(obj, prop_is_list_or_vo=False):
            if isinstance(obj, list):
                return [dictify(o, prop_is_list_or_vo) for o in obj]
            elif hasattr(obj, "to_dict"):
                return obj.to_dict(dirty=dirty, belongs_to_parent=prop_is_list_or_vo)
            else:
                return obj
        if "vendor_phone_number" == "type" or (self.vendor_phone_number is not self.__undef__ and (not (dirty and not self._vendor_phone_number[1]) or isinstance(self.vendor_phone_number, list) or belongs_to_parent)):
            dct["vendorPhoneNumber"] = dictify(self.vendor_phone_number)
        if "current_locale" == "type" or (self.current_locale is not self.__undef__ and (not (dirty and not self._current_locale[1]))):
            dct["currentLocale"] = dictify(self.current_locale)
        if "vendor_url" == "type" or (self.vendor_url is not self.__undef__ and (not (dirty and not self._vendor_url[1]) or isinstance(self.vendor_url, list) or belongs_to_parent)):
            dct["vendorURL"] = dictify(self.vendor_url)
        if "enabled_features" == "type" or (self.enabled_features is not self.__undef__ and (not (dirty and not self._enabled_features[1]))):
            dct["enabledFeatures"] = dictify(self.enabled_features)
        if "vendor_address" == "type" or (self.vendor_address is not self.__undef__ and (not (dirty and not self._vendor_address[1]) or isinstance(self.vendor_address, list) or belongs_to_parent)):
            dct["vendorAddress"] = dictify(self.vendor_address, prop_is_list_or_vo=True)
        if "build_version" == "type" or (self.build_version is not self.__undef__ and (not (dirty and not self._build_version[1]))):
            dct["buildVersion"] = dictify(self.build_version)
        if "engine_type" == "type" or (self.engine_type is not self.__undef__ and (not (dirty and not self._engine_type[1]) or isinstance(self.engine_type, list) or belongs_to_parent)):
            dct["engineType"] = dictify(self.engine_type)
        elif belongs_to_parent and self.engine_type is self.__undef__:
            dct["engineType"] = "UNSET"
        if "support_url" == "type" or (self.support_url is not self.__undef__ and (not (dirty and not self._support_url[1]) or isinstance(self.support_url, list) or belongs_to_parent)):
            dct["supportURL"] = dictify(self.support_url)
        if "support_contacts" == "type" or (self.support_contacts is not self.__undef__ and (not (dirty and not self._support_contacts[1]) or isinstance(self.support_contacts, list) or belongs_to_parent)):
            dct["supportContacts"] = dictify(self.support_contacts, prop_is_list_or_vo=True)
        if "api_version" == "type" or (self.api_version is not self.__undef__ and (not (dirty and not self._api_version[1]))):
            dct["apiVersion"] = dictify(self.api_version)
        if "vendor_email" == "type" or (self.vendor_email is not self.__undef__ and (not (dirty and not self._vendor_email[1]) or isinstance(self.vendor_email, list) or belongs_to_parent)):
            dct["vendorEmail"] = dictify(self.vendor_email)
        if "engine_qualifier" == "type" or (self.engine_qualifier is not self.__undef__ and (not (dirty and not self._engine_qualifier[1]) or isinstance(self.engine_qualifier, list) or belongs_to_parent)):
            dct["engineQualifier"] = dictify(self.engine_qualifier)
        if "build_timestamp" == "type" or (self.build_timestamp is not self.__undef__ and (not (dirty and not self._build_timestamp[1]))):
            dct["buildTimestamp"] = dictify(self.build_timestamp)
        if "product_type" == "type" or (self.product_type is not self.__undef__ and (not (dirty and not self._product_type[1]))):
            dct["productType"] = dictify(self.product_type)
        if "build_title" == "type" or (self.build_title is not self.__undef__ and (not (dirty and not self._build_title[1]))):
            dct["buildTitle"] = dictify(self.build_title)
        if "vendor_name" == "type" or (self.vendor_name is not self.__undef__ and (not (dirty and not self._vendor_name[1]) or isinstance(self.vendor_name, list) or belongs_to_parent)):
            dct["vendorName"] = dictify(self.vendor_name)
        if "kernel_name" == "type" or (self.kernel_name is not self.__undef__ and (not (dirty and not self._kernel_name[1]))):
            dct["kernelName"] = dictify(self.kernel_name)
        if "product_name" == "type" or (self.product_name is not self.__undef__ and (not (dirty and not self._product_name[1]))):
            dct["productName"] = dictify(self.product_name)
        if "banner" == "type" or (self.banner is not self.__undef__ and (not (dirty and not self._banner[1]))):
            dct["banner"] = dictify(self.banner)
        if "configured" == "type" or (self.configured is not self.__undef__ and (not (dirty and not self._configured[1]))):
            dct["configured"] = dictify(self.configured)
        if "locales" == "type" or (self.locales is not self.__undef__ and (not (dirty and not self._locales[1]))):
            dct["locales"] = dictify(self.locales)
        return dct

    def dirty(self):
        return self.from_dict(self.to_dict(dirty=False), dirty=True)

    def force_dirty(self):
        self._vendor_phone_number = (self._vendor_phone_number[0], True)
        self._current_locale = (self._current_locale[0], True)
        self._vendor_url = (self._vendor_url[0], True)
        self._enabled_features = (self._enabled_features[0], True)
        self._vendor_address = (self._vendor_address[0], True)
        self._build_version = (self._build_version[0], True)
        self._engine_type = (self._engine_type[0], True)
        self._support_url = (self._support_url[0], True)
        self._support_contacts = (self._support_contacts[0], True)
        self._api_version = (self._api_version[0], True)
        self._vendor_email = (self._vendor_email[0], True)
        self._engine_qualifier = (self._engine_qualifier[0], True)
        self._build_timestamp = (self._build_timestamp[0], True)
        self._product_type = (self._product_type[0], True)
        self._build_title = (self._build_title[0], True)
        self._vendor_name = (self._vendor_name[0], True)
        self._kernel_name = (self._kernel_name[0], True)
        self._product_name = (self._product_name[0], True)
        self._banner = (self._banner[0], True)
        self._configured = (self._configured[0], True)
        self._locales = (self._locales[0], True)

    def is_dirty(self):
        return any([self._vendor_phone_number[1], self._current_locale[1], self._vendor_url[1], self._enabled_features[1], self._vendor_address[1], self._build_version[1], self._engine_type[1], self._support_url[1], self._support_contacts[1], self._api_version[1], self._vendor_email[1], self._engine_qualifier[1], self._build_timestamp[1], self._product_type[1], self._build_title[1], self._vendor_name[1], self._kernel_name[1], self._product_name[1], self._banner[1], self._configured[1], self._locales[1]])

    def __eq__(self, other):
        if other is None:
            return False
        if not isinstance(other, PublicSystemInfo):
            return False
        return super(PublicSystemInfo, self).__eq__(other) and \
               self.vendor_phone_number == other.vendor_phone_number and \
               self.current_locale == other.current_locale and \
               self.vendor_url == other.vendor_url and \
               self.enabled_features == other.enabled_features and \
               self.vendor_address == other.vendor_address and \
               self.build_version == other.build_version and \
               self.engine_type == other.engine_type and \
               self.support_url == other.support_url and \
               self.support_contacts == other.support_contacts and \
               self.api_version == other.api_version and \
               self.vendor_email == other.vendor_email and \
               self.engine_qualifier == other.engine_qualifier and \
               self.build_timestamp == other.build_timestamp and \
               self.product_type == other.product_type and \
               self.build_title == other.build_title and \
               self.vendor_name == other.vendor_name and \
               self.kernel_name == other.kernel_name and \
               self.product_name == other.product_name and \
               self.banner == other.banner and \
               self.configured == other.configured and \
               self.locales == other.locales

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        return id(self)

    def __repr__(self):
        return common.generate_repr_string(self)

    @property
    def vendor_phone_number(self):
        """
        Corporate headquarters telephone number.

        :rtype: ``TEXT_TYPE``
        """
        return self._vendor_phone_number[0]

    @vendor_phone_number.setter
    def vendor_phone_number(self, value):
        self._vendor_phone_number = (value, True)

    @property
    def current_locale(self):
        """
        The current system locale.

        :rtype: ``TEXT_TYPE``
        """
        return self._current_locale[0]

    @current_locale.setter
    def current_locale(self, value):
        self._current_locale = (value, True)

    @property
    def vendor_url(self):
        """
        Corporate home page.

        :rtype: ``TEXT_TYPE``
        """
        return self._vendor_url[0]

    @vendor_url.setter
    def vendor_url(self, value):
        self._vendor_url = (value, True)

    @property
    def enabled_features(self):
        """
        The list of enabled features on this Delphix Engine.

        :rtype: ``list`` of ``TEXT_TYPE``
        """
        return self._enabled_features[0]

    @enabled_features.setter
    def enabled_features(self, value):
        self._enabled_features = (value, True)

    @property
    def vendor_address(self):
        """
        Address of vendor headquarters. Free form collection of strings to
        accomodate any region.

        :rtype: ``list`` of ``TEXT_TYPE``
        """
        return self._vendor_address[0]

    @vendor_address.setter
    def vendor_address(self, value):
        self._vendor_address = (value, True)

    @property
    def build_version(self):
        """
        Delphix version of the current system software.

        :rtype: :py:class:`v1_10_2.web.vo.VersionInfo`
        """
        return self._build_version[0]

    @build_version.setter
    def build_version(self, value):
        self._build_version = (value, True)

    @property
    def engine_type(self):
        """
        *(default value: UNSET)* Engine type, could be Masking or
        Virtualization. *(permitted values: VIRTUALIZATION, MASKING, BOTH,
        UNSET)*

        :rtype: ``TEXT_TYPE``
        """
        return self._engine_type[0]

    @engine_type.setter
    def engine_type(self, value):
        self._engine_type = (value, True)

    @property
    def support_url(self):
        """
        Technical Support URL.

        :rtype: ``TEXT_TYPE``
        """
        return self._support_url[0]

    @support_url.setter
    def support_url(self, value):
        self._support_url = (value, True)

    @property
    def support_contacts(self):
        """
        Technical support phone numbers.

        :rtype: ``list`` of :py:class:`v1_10_2.web.vo.SupportContact`
        """
        return self._support_contacts[0]

    @support_contacts.setter
    def support_contacts(self, value):
        self._support_contacts = (value, True)

    @property
    def api_version(self):
        """
        Maximum supported API version of the current system software.

        :rtype: :py:class:`v1_10_2.web.vo.APIVersion`
        """
        return self._api_version[0]

    @api_version.setter
    def api_version(self, value):
        self._api_version = (value, True)

    @property
    def vendor_email(self):
        """
        Corporate headquarters email address.

        :rtype: ``TEXT_TYPE``
        """
        return self._vendor_email[0]

    @vendor_email.setter
    def vendor_email(self, value):
        self._vendor_email = (value, True)

    @property
    def engine_qualifier(self):
        """
        Qualifier for referencing instances of (e.g. 'Delphix') engines in
        messages like 'The <engineQualifier> Engine failed to ...'.

        :rtype: ``TEXT_TYPE``
        """
        return self._engine_qualifier[0]

    @engine_qualifier.setter
    def engine_qualifier(self, value):
        self._engine_qualifier = (value, True)

    @property
    def build_timestamp(self):
        """
        Time at which the current system software was built.

        :rtype: ``TEXT_TYPE``
        """
        return self._build_timestamp[0]

    @build_timestamp.setter
    def build_timestamp(self, value):
        self._build_timestamp = (value, True)

    @property
    def product_type(self):
        """
        Product type.

        :rtype: ``TEXT_TYPE``
        """
        return self._product_type[0]

    @product_type.setter
    def product_type(self, value):
        self._product_type = (value, True)

    @property
    def build_title(self):
        """
        Description of the current system software.

        :rtype: ``TEXT_TYPE``
        """
        return self._build_title[0]

    @build_title.setter
    def build_title(self, value):
        self._build_title = (value, True)

    @property
    def vendor_name(self):
        """
        Vendor name, for use in messages like 'Please contact <vendorName>
        customer support'.

        :rtype: ``TEXT_TYPE``
        """
        return self._vendor_name[0]

    @vendor_name.setter
    def vendor_name(self, value):
        self._vendor_name = (value, True)

    @property
    def kernel_name(self):
        """
        The operating system kernel name.

        :rtype: ``TEXT_TYPE``
        """
        return self._kernel_name[0]

    @kernel_name.setter
    def kernel_name(self, value):
        self._kernel_name = (value, True)

    @property
    def product_name(self):
        """
        Name of the product that the system is running.

        :rtype: ``TEXT_TYPE``
        """
        return self._product_name[0]

    @product_name.setter
    def product_name(self, value):
        self._product_name = (value, True)

    @property
    def banner(self):
        """
        Security banner to display prior to login.

        :rtype: ``TEXT_TYPE``
        """
        return self._banner[0]

    @banner.setter
    def banner(self, value):
        self._banner = (value, True)

    @property
    def configured(self):
        """
        Indicates whether the server has gone through initial setup or not.

        :rtype: ``bool``
        """
        return self._configured[0]

    @configured.setter
    def configured(self, value):
        self._configured = (value, True)

    @property
    def locales(self):
        """
        List of available locales.

        :rtype: ``list`` of ``TEXT_TYPE``
        """
        return self._locales[0]

    @locales.setter
    def locales(self, value):
        self._locales = (value, True)




#
# Copyright 2019 by Delphix
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

#
# This class has been automatically generated from:
#     /delphix-oracle-live-source-runtime.json
#
# Do not edit this file manually!
#

from __future__ import unicode_literals

from delphixpy.v1_10_6.web.objects.OracleBaseSourceRuntime import OracleBaseSourceRuntime
from delphixpy.v1_10_6 import common

class __Undef(object):
    def __repr__(self):
        return "undef"

    def __setattr__(self, name, value):
        raise Exception('Cannot modify attributes of __Undef.')

_UNDEFINED = __Undef()

try:
    TEXT_TYPE = unicode
except NameError:
    TEXT_TYPE = str

class OracleLiveSourceRuntime(OracleBaseSourceRuntime):
    """
    *(extends* :py:class:`v1_10_6.web.vo.OracleBaseSourceRuntime` *)* Runtime
    (non-persistent) properties of an Oracle LiveSource.
    """
    def __init__(self, undef_enabled=True):
        super(OracleLiveSourceRuntime, self).__init__()
        self._type = ("OracleLiveSourceRuntime", True)
        self._source_resetlogs_id_change_detected = (self.__undef__, True)
        self._last_update_time = (self.__undef__, True)
        self._is_data_age_warning_exceeded = (self.__undef__, True)
        self._current_data_age = (self.__undef__, True)
        self._apply_status = (self.__undef__, True)
        self._unexpected_role_change_detected = (self.__undef__, True)
        self._transport_status = (self.__undef__, True)
        self._non_logged_data_detected = (self.__undef__, True)
        self._source_database_timezone = (self.__undef__, True)

    API_VERSION = "1.10.6"

    @classmethod
    def from_dict(cls, data, dirty=False, undef_enabled=True):
        obj = super(OracleLiveSourceRuntime, cls).from_dict(data, dirty, undef_enabled)
        obj._source_resetlogs_id_change_detected = (data.get("sourceResetlogsIDChangeDetected", obj.__undef__), dirty)
        if obj._source_resetlogs_id_change_detected[0] is not None and obj._source_resetlogs_id_change_detected[0] is not obj.__undef__:
            assert isinstance(obj._source_resetlogs_id_change_detected[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._source_resetlogs_id_change_detected[0], type(obj._source_resetlogs_id_change_detected[0])))
            common.validate_format(obj._source_resetlogs_id_change_detected[0], "None", None, None)
        obj._last_update_time = (data.get("lastUpdateTime", obj.__undef__), dirty)
        if obj._last_update_time[0] is not None and obj._last_update_time[0] is not obj.__undef__:
            assert isinstance(obj._last_update_time[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._last_update_time[0], type(obj._last_update_time[0])))
            common.validate_format(obj._last_update_time[0], "date", None, None)
        obj._is_data_age_warning_exceeded = (data.get("isDataAgeWarningExceeded", obj.__undef__), dirty)
        if obj._is_data_age_warning_exceeded[0] is not None and obj._is_data_age_warning_exceeded[0] is not obj.__undef__:
            assert isinstance(obj._is_data_age_warning_exceeded[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._is_data_age_warning_exceeded[0], type(obj._is_data_age_warning_exceeded[0])))
            common.validate_format(obj._is_data_age_warning_exceeded[0], "None", None, None)
        obj._current_data_age = (data.get("currentDataAge", obj.__undef__), dirty)
        if obj._current_data_age[0] is not None and obj._current_data_age[0] is not obj.__undef__:
            assert isinstance(obj._current_data_age[0], int), ("Expected one of [u'integer'], but got %s of type %s" % (obj._current_data_age[0], type(obj._current_data_age[0])))
            common.validate_format(obj._current_data_age[0], "None", None, None)
        obj._apply_status = (data.get("applyStatus", obj.__undef__), dirty)
        if obj._apply_status[0] is not None and obj._apply_status[0] is not obj.__undef__:
            assert isinstance(obj._apply_status[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._apply_status[0], type(obj._apply_status[0])))
            assert obj._apply_status[0] in [u'UNKNOWN', u'WORKING', u'APPLY_FAILED', u'APPLY_ON_WRONG_INCARNATION', u'UNRESOLVABLE_GAP_DETECTED'], "Expected enum [u'UNKNOWN', u'WORKING', u'APPLY_FAILED', u'APPLY_ON_WRONG_INCARNATION', u'UNRESOLVABLE_GAP_DETECTED'] but got %s" % obj._apply_status[0]
            common.validate_format(obj._apply_status[0], "None", None, None)
        obj._unexpected_role_change_detected = (data.get("unexpectedRoleChangeDetected", obj.__undef__), dirty)
        if obj._unexpected_role_change_detected[0] is not None and obj._unexpected_role_change_detected[0] is not obj.__undef__:
            assert isinstance(obj._unexpected_role_change_detected[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._unexpected_role_change_detected[0], type(obj._unexpected_role_change_detected[0])))
            common.validate_format(obj._unexpected_role_change_detected[0], "None", None, None)
        obj._transport_status = (data.get("transportStatus", obj.__undef__), dirty)
        if obj._transport_status[0] is not None and obj._transport_status[0] is not obj.__undef__:
            assert isinstance(obj._transport_status[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._transport_status[0], type(obj._transport_status[0])))
            assert obj._transport_status[0] in [u'UNKNOWN', u'WORKING', u'NO_INITIAL_DATA', u'NO_NEW_DATA'], "Expected enum [u'UNKNOWN', u'WORKING', u'NO_INITIAL_DATA', u'NO_NEW_DATA'] but got %s" % obj._transport_status[0]
            common.validate_format(obj._transport_status[0], "None", None, None)
        obj._non_logged_data_detected = (data.get("nonLoggedDataDetected", obj.__undef__), dirty)
        if obj._non_logged_data_detected[0] is not None and obj._non_logged_data_detected[0] is not obj.__undef__:
            assert isinstance(obj._non_logged_data_detected[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._non_logged_data_detected[0], type(obj._non_logged_data_detected[0])))
            common.validate_format(obj._non_logged_data_detected[0], "None", None, None)
        obj._source_database_timezone = (data.get("sourceDatabaseTimezone", obj.__undef__), dirty)
        if obj._source_database_timezone[0] is not None and obj._source_database_timezone[0] is not obj.__undef__:
            assert isinstance(obj._source_database_timezone[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._source_database_timezone[0], type(obj._source_database_timezone[0])))
            common.validate_format(obj._source_database_timezone[0], "None", None, None)
        return obj

    def to_dict(self, dirty=False, belongs_to_parent=False):
        dct = super(OracleLiveSourceRuntime, self).to_dict(dirty, belongs_to_parent)

        def dictify(obj, prop_is_list_or_vo=False):
            if isinstance(obj, list):
                return [dictify(o, prop_is_list_or_vo) for o in obj]
            elif hasattr(obj, "to_dict"):
                return obj.to_dict(dirty=dirty, belongs_to_parent=prop_is_list_or_vo)
            else:
                return obj
        if "source_resetlogs_id_change_detected" == "type" or (self.source_resetlogs_id_change_detected is not self.__undef__ and (not (dirty and not self._source_resetlogs_id_change_detected[1]))):
            dct["sourceResetlogsIDChangeDetected"] = dictify(self.source_resetlogs_id_change_detected)
        if "last_update_time" == "type" or (self.last_update_time is not self.__undef__ and (not (dirty and not self._last_update_time[1]))):
            dct["lastUpdateTime"] = dictify(self.last_update_time)
        if "is_data_age_warning_exceeded" == "type" or (self.is_data_age_warning_exceeded is not self.__undef__ and (not (dirty and not self._is_data_age_warning_exceeded[1]))):
            dct["isDataAgeWarningExceeded"] = dictify(self.is_data_age_warning_exceeded)
        if "current_data_age" == "type" or (self.current_data_age is not self.__undef__ and (not (dirty and not self._current_data_age[1]))):
            dct["currentDataAge"] = dictify(self.current_data_age)
        if "apply_status" == "type" or (self.apply_status is not self.__undef__ and (not (dirty and not self._apply_status[1]))):
            dct["applyStatus"] = dictify(self.apply_status)
        if "unexpected_role_change_detected" == "type" or (self.unexpected_role_change_detected is not self.__undef__ and (not (dirty and not self._unexpected_role_change_detected[1]))):
            dct["unexpectedRoleChangeDetected"] = dictify(self.unexpected_role_change_detected)
        if "transport_status" == "type" or (self.transport_status is not self.__undef__ and (not (dirty and not self._transport_status[1]))):
            dct["transportStatus"] = dictify(self.transport_status)
        if "non_logged_data_detected" == "type" or (self.non_logged_data_detected is not self.__undef__ and (not (dirty and not self._non_logged_data_detected[1]))):
            dct["nonLoggedDataDetected"] = dictify(self.non_logged_data_detected)
        if "source_database_timezone" == "type" or (self.source_database_timezone is not self.__undef__ and (not (dirty and not self._source_database_timezone[1]))):
            dct["sourceDatabaseTimezone"] = dictify(self.source_database_timezone)
        return dct

    def dirty(self):
        return self.from_dict(self.to_dict(dirty=False), dirty=True)

    def force_dirty(self):
        self._source_resetlogs_id_change_detected = (self._source_resetlogs_id_change_detected[0], True)
        self._last_update_time = (self._last_update_time[0], True)
        self._is_data_age_warning_exceeded = (self._is_data_age_warning_exceeded[0], True)
        self._current_data_age = (self._current_data_age[0], True)
        self._apply_status = (self._apply_status[0], True)
        self._unexpected_role_change_detected = (self._unexpected_role_change_detected[0], True)
        self._transport_status = (self._transport_status[0], True)
        self._non_logged_data_detected = (self._non_logged_data_detected[0], True)
        self._source_database_timezone = (self._source_database_timezone[0], True)

    def is_dirty(self):
        return any([self._source_resetlogs_id_change_detected[1], self._last_update_time[1], self._is_data_age_warning_exceeded[1], self._current_data_age[1], self._apply_status[1], self._unexpected_role_change_detected[1], self._transport_status[1], self._non_logged_data_detected[1], self._source_database_timezone[1]])

    def __eq__(self, other):
        if other is None:
            return False
        if not isinstance(other, OracleLiveSourceRuntime):
            return False
        return super(OracleLiveSourceRuntime, self).__eq__(other) and \
               self.source_resetlogs_id_change_detected == other.source_resetlogs_id_change_detected and \
               self.last_update_time == other.last_update_time and \
               self.is_data_age_warning_exceeded == other.is_data_age_warning_exceeded and \
               self.current_data_age == other.current_data_age and \
               self.apply_status == other.apply_status and \
               self.unexpected_role_change_detected == other.unexpected_role_change_detected and \
               self.transport_status == other.transport_status and \
               self.non_logged_data_detected == other.non_logged_data_detected and \
               self.source_database_timezone == other.source_database_timezone

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        return id(self)

    def __repr__(self):
        return common.generate_repr_string(self)

    @property
    def source_resetlogs_id_change_detected(self):
        """
        Indicates whether the incarnation ID changed on the primary database.

        :rtype: ``bool``
        """
        return self._source_resetlogs_id_change_detected[0]

    @source_resetlogs_id_change_detected.setter
    def source_resetlogs_id_change_detected(self, value):
        self._source_resetlogs_id_change_detected = (value, True)

    @property
    def last_update_time(self):
        """
        The time at which this runtime data was updated.

        :rtype: ``TEXT_TYPE``
        """
        return self._last_update_time[0]

    @last_update_time.setter
    def last_update_time(self, value):
        self._last_update_time = (value, True)

    @property
    def is_data_age_warning_exceeded(self):
        """
        Has data age exceeded the user specified threshold.

        :rtype: ``bool``
        """
        return self._is_data_age_warning_exceeded[0]

    @is_data_age_warning_exceeded.setter
    def is_data_age_warning_exceeded(self, value):
        self._is_data_age_warning_exceeded = (value, True)

    @property
    def current_data_age(self):
        """
        Current data lag between LiveSource and source database in seconds.

        :rtype: ``int``
        """
        return self._current_data_age[0]

    @current_data_age.setter
    def current_data_age(self, value):
        self._current_data_age = (value, True)

    @property
    def apply_status(self):
        """
        MRP apply status for the standby database associated with the
        LiveSource. *(permitted values: UNKNOWN, WORKING, APPLY_FAILED,
        APPLY_ON_WRONG_INCARNATION, UNRESOLVABLE_GAP_DETECTED)*

        :rtype: ``TEXT_TYPE``
        """
        return self._apply_status[0]

    @apply_status.setter
    def apply_status(self, value):
        self._apply_status = (value, True)

    @property
    def unexpected_role_change_detected(self):
        """
        Indicates whether the LiveSource is not in standby mode.

        :rtype: ``bool``
        """
        return self._unexpected_role_change_detected[0]

    @unexpected_role_change_detected.setter
    def unexpected_role_change_detected(self, value):
        self._unexpected_role_change_detected = (value, True)

    @property
    def transport_status(self):
        """
        Redo log transport status from the source database to the LiveSource.
        *(permitted values: UNKNOWN, WORKING, NO_INITIAL_DATA, NO_NEW_DATA)*

        :rtype: ``TEXT_TYPE``
        """
        return self._transport_status[0]

    @transport_status.setter
    def transport_status(self, value):
        self._transport_status = (value, True)

    @property
    def non_logged_data_detected(self):
        """
        Indicates whether there is non-logged data on the standby.

        :rtype: ``bool``
        """
        return self._non_logged_data_detected[0]

    @non_logged_data_detected.setter
    def non_logged_data_detected(self, value):
        self._non_logged_data_detected = (value, True)

    @property
    def source_database_timezone(self):
        """
        Time zone of the source database at the time the runtime data was
        updated.

        :rtype: ``TEXT_TYPE``
        """
        return self._source_database_timezone[0]

    @source_database_timezone.setter
    def source_database_timezone(self, value):
        self._source_database_timezone = (value, True)




#
# Copyright 2019 by Delphix
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

#
# This class has been automatically generated from:
#     /delphix-syslog-config.json
#
# Do not edit this file manually!
#

from __future__ import unicode_literals

from delphixpy.v1_7_0.web.objects.TypedObject import TypedObject
from delphixpy.v1_7_0 import factory
from delphixpy.v1_7_0 import common

class __Undef(object):
    def __repr__(self):
        return "undef"

    def __setattr__(self, name, value):
        raise Exception('Cannot modify attributes of __Undef.')

_UNDEFINED = __Undef()

try:
    TEXT_TYPE = unicode
except NameError:
    TEXT_TYPE = str

class SyslogConfig(TypedObject):
    """
    *(extends* :py:class:`v1_7_0.web.vo.TypedObject` *)* Syslog configuration.
    """
    def __init__(self, undef_enabled=True):
        super(SyslogConfig, self).__init__()
        self._type = ("SyslogConfig", True)
        self._pattern = (self.__undef__, True)
        self._format = (self.__undef__, True)
        self._enabled = (self.__undef__, True)
        self._severity = (self.__undef__, True)
        self._servers = (self.__undef__, True)

    API_VERSION = "1.7.0"

    @classmethod
    def from_dict(cls, data, dirty=False, undef_enabled=True):
        obj = super(SyslogConfig, cls).from_dict(data, dirty, undef_enabled)
        obj._pattern = (data.get("pattern", obj.__undef__), dirty)
        if obj._pattern[0] is not None and obj._pattern[0] is not obj.__undef__:
            assert isinstance(obj._pattern[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._pattern[0], type(obj._pattern[0])))
            common.validate_format(obj._pattern[0], "None", None, None)
        obj._format = (data.get("format", obj.__undef__), dirty)
        if obj._format[0] is not None and obj._format[0] is not obj.__undef__:
            assert isinstance(obj._format[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._format[0], type(obj._format[0])))
            assert obj._format[0] in [u'TEXT', u'JSON'], "Expected enum [u'TEXT', u'JSON'] but got %s" % obj._format[0]
            common.validate_format(obj._format[0], "None", None, None)
        obj._enabled = (data.get("enabled", obj.__undef__), dirty)
        if obj._enabled[0] is not None and obj._enabled[0] is not obj.__undef__:
            assert isinstance(obj._enabled[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._enabled[0], type(obj._enabled[0])))
            common.validate_format(obj._enabled[0], "None", None, None)
        obj._severity = (data.get("severity", obj.__undef__), dirty)
        if obj._severity[0] is not None and obj._severity[0] is not obj.__undef__:
            assert isinstance(obj._severity[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._severity[0], type(obj._severity[0])))
            assert obj._severity[0] in [u'EMERGENCY', u'ALERT', u'CRITICAL', u'ERROR', u'WARNING', u'NOTICE', u'INFORMATIONAL', u'DEBUG'], "Expected enum [u'EMERGENCY', u'ALERT', u'CRITICAL', u'ERROR', u'WARNING', u'NOTICE', u'INFORMATIONAL', u'DEBUG'] but got %s" % obj._severity[0]
            common.validate_format(obj._severity[0], "None", None, None)
        obj._servers = []
        for item in data.get("servers") or []:
            obj._servers.append(factory.create_object(item))
            factory.validate_type(obj._servers[-1], "SyslogServer")
        obj._servers = (obj._servers, dirty)
        return obj

    def to_dict(self, dirty=False, belongs_to_parent=False):
        dct = super(SyslogConfig, self).to_dict(dirty, belongs_to_parent)

        def dictify(obj, prop_is_list_or_vo=False):
            if isinstance(obj, list):
                return [dictify(o, prop_is_list_or_vo) for o in obj]
            elif hasattr(obj, "to_dict"):
                return obj.to_dict(dirty=dirty, belongs_to_parent=prop_is_list_or_vo)
            else:
                return obj
        if "pattern" == "type" or (self.pattern is not self.__undef__ and (not (dirty and not self._pattern[1]) or isinstance(self.pattern, list) or belongs_to_parent)):
            dct["pattern"] = dictify(self.pattern)
        elif belongs_to_parent and self.pattern is self.__undef__:
            dct["pattern"] = "%-5p delphix : %m%n"
        if "format" == "type" or (self.format is not self.__undef__ and (not (dirty and not self._format[1]) or isinstance(self.format, list) or belongs_to_parent)):
            dct["format"] = dictify(self.format)
        elif belongs_to_parent and self.format is self.__undef__:
            dct["format"] = "TEXT"
        if "enabled" == "type" or (self.enabled is not self.__undef__ and (not (dirty and not self._enabled[1]) or isinstance(self.enabled, list) or belongs_to_parent)):
            dct["enabled"] = dictify(self.enabled)
        elif belongs_to_parent and self.enabled is self.__undef__:
            dct["enabled"] = True
        if "severity" == "type" or (self.severity is not self.__undef__ and (not (dirty and not self._severity[1]) or isinstance(self.severity, list) or belongs_to_parent)):
            dct["severity"] = dictify(self.severity)
        elif belongs_to_parent and self.severity is self.__undef__:
            dct["severity"] = "WARNING"
        if "servers" == "type" or (self.servers is not self.__undef__ and (not (dirty and not self._servers[1]) or isinstance(self.servers, list) or belongs_to_parent)):
            dct["servers"] = dictify(self.servers, prop_is_list_or_vo=True)
        return dct

    def dirty(self):
        return self.from_dict(self.to_dict(dirty=False), dirty=True)

    def force_dirty(self):
        self._pattern = (self._pattern[0], True)
        self._format = (self._format[0], True)
        self._enabled = (self._enabled[0], True)
        self._severity = (self._severity[0], True)
        self._servers = (self._servers[0], True)

    def is_dirty(self):
        return any([self._pattern[1], self._format[1], self._enabled[1], self._severity[1], self._servers[1]])

    def __eq__(self, other):
        if other is None:
            return False
        if not isinstance(other, SyslogConfig):
            return False
        return super(SyslogConfig, self).__eq__(other) and \
               self.pattern == other.pattern and \
               self.format == other.format and \
               self.enabled == other.enabled and \
               self.severity == other.severity and \
               self.servers == other.servers

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        return id(self)

    def __repr__(self):
        return common.generate_repr_string(self)

    @property
    def pattern(self):
        """
        *(default value: %-5p delphix : %m%n)* Syslog logging pattern. Events
        will be logged in the pattern as specified.

        :rtype: ``TEXT_TYPE``
        """
        return self._pattern[0]

    @pattern.setter
    def pattern(self, value):
        self._pattern = (value, True)

    @property
    def format(self):
        """
        *(default value: TEXT)* Syslog message format. *(permitted values:
        TEXT, JSON)*

        :rtype: ``TEXT_TYPE``
        """
        return self._format[0]

    @format.setter
    def format(self, value):
        self._format = (value, True)

    @property
    def enabled(self):
        """
        *(default value: True)* True if the syslog service is enabled.

        :rtype: ``bool``
        """
        return self._enabled[0]

    @enabled.setter
    def enabled(self, value):
        self._enabled = (value, True)

    @property
    def severity(self):
        """
        *(default value: WARNING)* Syslog logging severity. Only events at or
        above this severity will be logged. *(permitted values: EMERGENCY,
        ALERT, CRITICAL, ERROR, WARNING, NOTICE, INFORMATIONAL, DEBUG)*

        :rtype: ``TEXT_TYPE``
        """
        return self._severity[0]

    @severity.setter
    def severity(self, value):
        self._severity = (value, True)

    @property
    def servers(self):
        """
        List of syslog servers. At least one syslog server must be specified.

        :rtype: ``list`` of :py:class:`v1_7_0.web.vo.SyslogServer`
        """
        return self._servers[0]

    @servers.setter
    def servers(self, value):
        self._servers = (value, True)




#
# Copyright 2019 by Delphix
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

#
# This class has been automatically generated from:
#     /delphix-mssql-commvault-config.json
#
# Do not edit this file manually!
#

from __future__ import unicode_literals

from delphixpy.v1_10_6.web.objects.TypedObject import TypedObject
from delphixpy.v1_10_6 import common

class __Undef(object):
    def __repr__(self):
        return "undef"

    def __setattr__(self, name, value):
        raise Exception('Cannot modify attributes of __Undef.')

_UNDEFINED = __Undef()

try:
    TEXT_TYPE = unicode
except NameError:
    TEXT_TYPE = str

class MSSqlCommvaultConfig(TypedObject):
    """
    *(extends* :py:class:`v1_10_6.web.vo.TypedObject` *)* MSSql Commvault
    configuration.
    """
    def __init__(self, undef_enabled=True):
        super(MSSqlCommvaultConfig, self).__init__()
        self._type = ("MSSqlCommvaultConfig", True)
        self._config_params = (self.__undef__, True)
        self._staging_client_name = (self.__undef__, True)
        self._commserve_host_name = (self.__undef__, True)
        self._config_template = (self.__undef__, True)
        self._source_client_name = (self.__undef__, True)

    API_VERSION = "1.10.6"

    @classmethod
    def from_dict(cls, data, dirty=False, undef_enabled=True):
        obj = super(MSSqlCommvaultConfig, cls).from_dict(data, dirty, undef_enabled)
        obj._config_params = (data.get("configParams", obj.__undef__), dirty)
        if obj._config_params[0] is not None and obj._config_params[0] is not obj.__undef__:
            assert isinstance(obj._config_params[0], dict), ("Expected one of [u'object'], but got %s of type %s" % (obj._config_params[0], type(obj._config_params[0])))
            common.validate_format(obj._config_params[0], "None", None, None)
        obj._staging_client_name = (data.get("stagingClientName", obj.__undef__), dirty)
        if obj._staging_client_name[0] is not None and obj._staging_client_name[0] is not obj.__undef__:
            assert isinstance(obj._staging_client_name[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._staging_client_name[0], type(obj._staging_client_name[0])))
            common.validate_format(obj._staging_client_name[0], "None", None, 256)
        obj._commserve_host_name = (data.get("commserveHostName", obj.__undef__), dirty)
        if obj._commserve_host_name[0] is not None and obj._commserve_host_name[0] is not obj.__undef__:
            assert isinstance(obj._commserve_host_name[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._commserve_host_name[0], type(obj._commserve_host_name[0])))
            common.validate_format(obj._commserve_host_name[0], "None", None, 256)
        obj._config_template = (data.get("configTemplate", obj.__undef__), dirty)
        if obj._config_template[0] is not None and obj._config_template[0] is not obj.__undef__:
            assert isinstance(obj._config_template[0], TEXT_TYPE), ("Expected one of [u'string', u'null'], but got %s of type %s" % (obj._config_template[0], type(obj._config_template[0])))
            common.validate_format(obj._config_template[0], "objectReference", None, None)
        obj._source_client_name = (data.get("sourceClientName", obj.__undef__), dirty)
        if obj._source_client_name[0] is not None and obj._source_client_name[0] is not obj.__undef__:
            assert isinstance(obj._source_client_name[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._source_client_name[0], type(obj._source_client_name[0])))
            common.validate_format(obj._source_client_name[0], "None", None, 256)
        return obj

    def to_dict(self, dirty=False, belongs_to_parent=False):
        dct = super(MSSqlCommvaultConfig, self).to_dict(dirty, belongs_to_parent)

        def dictify(obj, prop_is_list_or_vo=False):
            if isinstance(obj, list):
                return [dictify(o, prop_is_list_or_vo) for o in obj]
            elif hasattr(obj, "to_dict"):
                return obj.to_dict(dirty=dirty, belongs_to_parent=prop_is_list_or_vo)
            else:
                return obj
        if "config_params" == "type" or (self.config_params is not self.__undef__ and (not (dirty and not self._config_params[1]) or isinstance(self.config_params, list) or belongs_to_parent)):
            dct["configParams"] = dictify(self.config_params, prop_is_list_or_vo=True)
        if "staging_client_name" == "type" or (self.staging_client_name is not self.__undef__ and (not (dirty and not self._staging_client_name[1]) or isinstance(self.staging_client_name, list) or belongs_to_parent)):
            dct["stagingClientName"] = dictify(self.staging_client_name)
        if "commserve_host_name" == "type" or (self.commserve_host_name is not self.__undef__ and (not (dirty and not self._commserve_host_name[1]) or isinstance(self.commserve_host_name, list) or belongs_to_parent)):
            dct["commserveHostName"] = dictify(self.commserve_host_name)
        if "config_template" == "type" or (self.config_template is not self.__undef__ and (not (dirty and not self._config_template[1]) or isinstance(self.config_template, list) or belongs_to_parent)):
            dct["configTemplate"] = dictify(self.config_template)
        if "source_client_name" == "type" or (self.source_client_name is not self.__undef__ and (not (dirty and not self._source_client_name[1]) or isinstance(self.source_client_name, list) or belongs_to_parent)):
            dct["sourceClientName"] = dictify(self.source_client_name)
        return dct

    def dirty(self):
        return self.from_dict(self.to_dict(dirty=False), dirty=True)

    def force_dirty(self):
        self._config_params = (self._config_params[0], True)
        self._staging_client_name = (self._staging_client_name[0], True)
        self._commserve_host_name = (self._commserve_host_name[0], True)
        self._config_template = (self._config_template[0], True)
        self._source_client_name = (self._source_client_name[0], True)

    def is_dirty(self):
        return any([self._config_params[1], self._staging_client_name[1], self._commserve_host_name[1], self._config_template[1], self._source_client_name[1]])

    def __eq__(self, other):
        if other is None:
            return False
        if not isinstance(other, MSSqlCommvaultConfig):
            return False
        return super(MSSqlCommvaultConfig, self).__eq__(other) and \
               self.config_params == other.config_params and \
               self.staging_client_name == other.staging_client_name and \
               self.commserve_host_name == other.commserve_host_name and \
               self.config_template == other.config_template and \
               self.source_client_name == other.source_client_name

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        return id(self)

    def __repr__(self):
        return common.generate_repr_string(self)

    @property
    def config_params(self):
        """
        Commvault configuration parameter overrides.

        :rtype: ``dict``
        """
        return self._config_params[0]

    @config_params.setter
    def config_params(self, value):
        self._config_params = (value, True)

    @property
    def staging_client_name(self):
        """
        The staging client name of this Commvault configuration.

        :rtype: ``TEXT_TYPE``
        """
        return self._staging_client_name[0]

    @staging_client_name.setter
    def staging_client_name(self, value):
        self._staging_client_name = (value, True)

    @property
    def commserve_host_name(self):
        """
        The commserve host name of this Commvault configuration.

        :rtype: ``TEXT_TYPE``
        """
        return self._commserve_host_name[0]

    @commserve_host_name.setter
    def commserve_host_name(self, value):
        self._commserve_host_name = (value, True)

    @property
    def config_template(self):
        """
        Optional config template selection for Commvault configurations. If
        set, configParams will be ignored.

        :rtype: ``TEXT_TYPE`` *or* ``null``
        """
        return self._config_template[0]

    @config_template.setter
    def config_template(self, value):
        self._config_template = (value, True)

    @property
    def source_client_name(self):
        """
        The source client name of this Commvault configuration.

        :rtype: ``TEXT_TYPE``
        """
        return self._source_client_name[0]

    @source_client_name.setter
    def source_client_name(self, value):
        self._source_client_name = (value, True)




#
# Copyright 2019 by Delphix
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

#
# This class has been automatically generated from:
#     /delphix-network-interface.json
#
# Do not edit this file manually!
#

from __future__ import unicode_literals

from delphixpy.v1_10_4.web.objects.UserObject import UserObject
from delphixpy.v1_10_4 import factory
from delphixpy.v1_10_4 import common

class __Undef(object):
    def __repr__(self):
        return "undef"

    def __setattr__(self, name, value):
        raise Exception('Cannot modify attributes of __Undef.')

_UNDEFINED = __Undef()

try:
    TEXT_TYPE = unicode
except NameError:
    TEXT_TYPE = str

class NetworkInterface(UserObject):
    """
    *(extends* :py:class:`v1_10_4.web.vo.UserObject` *)* Configuration of an IP
    interface.
    """
    def __init__(self, undef_enabled=True):
        super(NetworkInterface, self).__init__()
        self._type = ("NetworkInterface", True)
        self._mac_address = (self.__undef__, True)
        self._addresses = (self.__undef__, True)
        self._mtu = (self.__undef__, True)
        self._state = (self.__undef__, True)
        self._device = (self.__undef__, True)
        self._mtu_range = (self.__undef__, True)

    API_VERSION = "1.10.4"

    @classmethod
    def from_dict(cls, data, dirty=False, undef_enabled=True):
        obj = super(NetworkInterface, cls).from_dict(data, dirty, undef_enabled)
        obj._mac_address = (data.get("macAddress", obj.__undef__), dirty)
        if obj._mac_address[0] is not None and obj._mac_address[0] is not obj.__undef__:
            assert isinstance(obj._mac_address[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._mac_address[0], type(obj._mac_address[0])))
            common.validate_format(obj._mac_address[0], "macAddress", None, None)
        obj._addresses = []
        for item in data.get("addresses") or []:
            obj._addresses.append(factory.create_object(item))
            factory.validate_type(obj._addresses[-1], "InterfaceAddress")
        obj._addresses = (obj._addresses, dirty)
        obj._mtu = (data.get("mtu", obj.__undef__), dirty)
        if obj._mtu[0] is not None and obj._mtu[0] is not obj.__undef__:
            assert isinstance(obj._mtu[0], int), ("Expected one of [u'integer'], but got %s of type %s" % (obj._mtu[0], type(obj._mtu[0])))
            common.validate_format(obj._mtu[0], "None", None, None)
        obj._state = (data.get("state", obj.__undef__), dirty)
        if obj._state[0] is not None and obj._state[0] is not obj.__undef__:
            assert isinstance(obj._state[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._state[0], type(obj._state[0])))
            assert obj._state[0] in [u'OK', u'DOWN', u'FAILED'], "Expected enum [u'OK', u'DOWN', u'FAILED'] but got %s" % obj._state[0]
            common.validate_format(obj._state[0], "None", None, None)
        obj._device = (data.get("device", obj.__undef__), dirty)
        if obj._device[0] is not None and obj._device[0] is not obj.__undef__:
            assert isinstance(obj._device[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._device[0], type(obj._device[0])))
            common.validate_format(obj._device[0], "None", None, None)
        obj._mtu_range = (data.get("mtuRange", obj.__undef__), dirty)
        if obj._mtu_range[0] is not None and obj._mtu_range[0] is not obj.__undef__:
            assert isinstance(obj._mtu_range[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._mtu_range[0], type(obj._mtu_range[0])))
            common.validate_format(obj._mtu_range[0], "None", None, None)
        return obj

    def to_dict(self, dirty=False, belongs_to_parent=False):
        dct = super(NetworkInterface, self).to_dict(dirty, belongs_to_parent)

        def dictify(obj, prop_is_list_or_vo=False):
            if isinstance(obj, list):
                return [dictify(o, prop_is_list_or_vo) for o in obj]
            elif hasattr(obj, "to_dict"):
                return obj.to_dict(dirty=dirty, belongs_to_parent=prop_is_list_or_vo)
            else:
                return obj
        if "mac_address" == "type" or (self.mac_address is not self.__undef__ and (not (dirty and not self._mac_address[1]))):
            dct["macAddress"] = dictify(self.mac_address)
        if "addresses" == "type" or (self.addresses is not self.__undef__ and (not (dirty and not self._addresses[1]) or isinstance(self.addresses, list) or belongs_to_parent)):
            dct["addresses"] = dictify(self.addresses, prop_is_list_or_vo=True)
        if "mtu" == "type" or (self.mtu is not self.__undef__ and (not (dirty and not self._mtu[1]) or isinstance(self.mtu, list) or belongs_to_parent)):
            dct["mtu"] = dictify(self.mtu)
        if "state" == "type" or (self.state is not self.__undef__ and (not (dirty and not self._state[1]))):
            dct["state"] = dictify(self.state)
        if "device" == "type" or (self.device is not self.__undef__ and (not (dirty and not self._device[1]))):
            dct["device"] = dictify(self.device)
        if "mtu_range" == "type" or (self.mtu_range is not self.__undef__ and (not (dirty and not self._mtu_range[1]))):
            dct["mtuRange"] = dictify(self.mtu_range)
        return dct

    def dirty(self):
        return self.from_dict(self.to_dict(dirty=False), dirty=True)

    def force_dirty(self):
        self._mac_address = (self._mac_address[0], True)
        self._addresses = (self._addresses[0], True)
        self._mtu = (self._mtu[0], True)
        self._state = (self._state[0], True)
        self._device = (self._device[0], True)
        self._mtu_range = (self._mtu_range[0], True)

    def is_dirty(self):
        return any([self._mac_address[1], self._addresses[1], self._mtu[1], self._state[1], self._device[1], self._mtu_range[1]])

    def __eq__(self, other):
        if other is None:
            return False
        if not isinstance(other, NetworkInterface):
            return False
        return super(NetworkInterface, self).__eq__(other) and \
               self.mac_address == other.mac_address and \
               self.addresses == other.addresses and \
               self.mtu == other.mtu and \
               self.state == other.state and \
               self.device == other.device and \
               self.mtu_range == other.mtu_range

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        return id(self)

    def __repr__(self):
        return common.generate_repr_string(self)

    @property
    def mac_address(self):
        """
        The MAC address associated with this interface.

        :rtype: ``TEXT_TYPE``
        """
        return self._mac_address[0]

    @mac_address.setter
    def mac_address(self, value):
        self._mac_address = (value, True)

    @property
    def addresses(self):
        """
        List of IP addresses assigned to the interface.

        :rtype: ``list`` of :py:class:`v1_10_4.web.vo.InterfaceAddress`
        """
        return self._addresses[0]

    @addresses.setter
    def addresses(self, value):
        self._addresses = (value, True)

    @property
    def mtu(self):
        """
        The maximum transmission unit for this interface.

        :rtype: ``int``
        """
        return self._mtu[0]

    @mtu.setter
    def mtu(self, value):
        self._mtu = (value, True)

    @property
    def state(self):
        """
        The state of the interface. *(permitted values: OK, DOWN, FAILED)*

        :rtype: ``TEXT_TYPE``
        """
        return self._state[0]

    @state.setter
    def state(self, value):
        self._state = (value, True)

    @property
    def device(self):
        """
        The name of the device over which this interface is configured.

        :rtype: ``TEXT_TYPE``
        """
        return self._device[0]

    @device.setter
    def device(self, value):
        self._device = (value, True)

    @property
    def mtu_range(self):
        """
        The range of possible values for the mtu property.

        :rtype: ``TEXT_TYPE``
        """
        return self._mtu_range[0]

    @mtu_range.setter
    def mtu_range(self, value):
        self._mtu_range = (value, True)




#
# Copyright 2019 by Delphix
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

#
# This class has been automatically generated from:
#     /delphix-toolkit-linked-staged-source.json
#
# Do not edit this file manually!
#

from __future__ import unicode_literals

from delphixpy.v1_5_0.web.objects.ToolkitLinkedSource import ToolkitLinkedSource
from delphixpy.v1_5_0 import common

class __Undef(object):
    def __repr__(self):
        return "undef"

    def __setattr__(self, name, value):
        raise Exception('Cannot modify attributes of __Undef.')

_UNDEFINED = __Undef()

try:
    TEXT_TYPE = unicode
except NameError:
    TEXT_TYPE = str

class ToolkitLinkedStagedSource(ToolkitLinkedSource):
    """
    *(extends* :py:class:`v1_5_0.web.vo.ToolkitLinkedSource` *)* A linked
    staged source definition for toolkits.
    """
    def __init__(self, undef_enabled=True):
        super(ToolkitLinkedStagedSource, self).__init__()
        self._type = ("ToolkitLinkedStagedSource", True)
        self._stop_staging = (self.__undef__, True)
        self._start_staging = (self.__undef__, True)
        self._resync = (self.__undef__, True)
        self._pre_snapshot = (self.__undef__, True)
        self._post_snapshot = (self.__undef__, True)
        self._status_script = (self.__undef__, True)

    API_VERSION = "1.5.0"

    @classmethod
    def from_dict(cls, data, dirty=False, undef_enabled=True):
        obj = super(ToolkitLinkedStagedSource, cls).from_dict(data, dirty, undef_enabled)
        if "stopStaging" not in data:
            raise ValueError("Missing required property \"stopStaging\".")
        obj._stop_staging = (data.get("stopStaging", obj.__undef__), dirty)
        if obj._stop_staging[0] is not None and obj._stop_staging[0] is not obj.__undef__:
            assert isinstance(obj._stop_staging[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._stop_staging[0], type(obj._stop_staging[0])))
            common.validate_format(obj._stop_staging[0], "None", None, None)
        if "startStaging" not in data:
            raise ValueError("Missing required property \"startStaging\".")
        obj._start_staging = (data.get("startStaging", obj.__undef__), dirty)
        if obj._start_staging[0] is not None and obj._start_staging[0] is not obj.__undef__:
            assert isinstance(obj._start_staging[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._start_staging[0], type(obj._start_staging[0])))
            common.validate_format(obj._start_staging[0], "None", None, None)
        if "resync" not in data:
            raise ValueError("Missing required property \"resync\".")
        obj._resync = (data.get("resync", obj.__undef__), dirty)
        if obj._resync[0] is not None and obj._resync[0] is not obj.__undef__:
            assert isinstance(obj._resync[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._resync[0], type(obj._resync[0])))
            common.validate_format(obj._resync[0], "None", None, None)
        if "preSnapshot" not in data:
            raise ValueError("Missing required property \"preSnapshot\".")
        obj._pre_snapshot = (data.get("preSnapshot", obj.__undef__), dirty)
        if obj._pre_snapshot[0] is not None and obj._pre_snapshot[0] is not obj.__undef__:
            assert isinstance(obj._pre_snapshot[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._pre_snapshot[0], type(obj._pre_snapshot[0])))
            common.validate_format(obj._pre_snapshot[0], "None", None, None)
        if "postSnapshot" not in data:
            raise ValueError("Missing required property \"postSnapshot\".")
        obj._post_snapshot = (data.get("postSnapshot", obj.__undef__), dirty)
        if obj._post_snapshot[0] is not None and obj._post_snapshot[0] is not obj.__undef__:
            assert isinstance(obj._post_snapshot[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._post_snapshot[0], type(obj._post_snapshot[0])))
            common.validate_format(obj._post_snapshot[0], "None", None, None)
        obj._status_script = (data.get("statusScript", obj.__undef__), dirty)
        if obj._status_script[0] is not None and obj._status_script[0] is not obj.__undef__:
            assert isinstance(obj._status_script[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._status_script[0], type(obj._status_script[0])))
            common.validate_format(obj._status_script[0], "None", None, None)
        return obj

    def to_dict(self, dirty=False, belongs_to_parent=False):
        dct = super(ToolkitLinkedStagedSource, self).to_dict(dirty, belongs_to_parent)

        def dictify(obj, prop_is_list_or_vo=False):
            if isinstance(obj, list):
                return [dictify(o, prop_is_list_or_vo) for o in obj]
            elif hasattr(obj, "to_dict"):
                return obj.to_dict(dirty=dirty, belongs_to_parent=prop_is_list_or_vo)
            else:
                return obj
        if "stop_staging" == "type" or (self.stop_staging is not self.__undef__ and (not (dirty and not self._stop_staging[1]) or isinstance(self.stop_staging, list) or belongs_to_parent)):
            dct["stopStaging"] = dictify(self.stop_staging)
        if "start_staging" == "type" or (self.start_staging is not self.__undef__ and (not (dirty and not self._start_staging[1]) or isinstance(self.start_staging, list) or belongs_to_parent)):
            dct["startStaging"] = dictify(self.start_staging)
        if "resync" == "type" or (self.resync is not self.__undef__ and (not (dirty and not self._resync[1]) or isinstance(self.resync, list) or belongs_to_parent)):
            dct["resync"] = dictify(self.resync)
        if "pre_snapshot" == "type" or (self.pre_snapshot is not self.__undef__ and (not (dirty and not self._pre_snapshot[1]) or isinstance(self.pre_snapshot, list) or belongs_to_parent)):
            dct["preSnapshot"] = dictify(self.pre_snapshot)
        if "post_snapshot" == "type" or (self.post_snapshot is not self.__undef__ and (not (dirty and not self._post_snapshot[1]) or isinstance(self.post_snapshot, list) or belongs_to_parent)):
            dct["postSnapshot"] = dictify(self.post_snapshot)
        if "status_script" == "type" or (self.status_script is not self.__undef__ and (not (dirty and not self._status_script[1]))):
            dct["statusScript"] = dictify(self.status_script)
        return dct

    def dirty(self):
        return self.from_dict(self.to_dict(dirty=False), dirty=True)

    def force_dirty(self):
        self._stop_staging = (self._stop_staging[0], True)
        self._start_staging = (self._start_staging[0], True)
        self._resync = (self._resync[0], True)
        self._pre_snapshot = (self._pre_snapshot[0], True)
        self._post_snapshot = (self._post_snapshot[0], True)
        self._status_script = (self._status_script[0], True)

    def is_dirty(self):
        return any([self._stop_staging[1], self._start_staging[1], self._resync[1], self._pre_snapshot[1], self._post_snapshot[1], self._status_script[1]])

    def __eq__(self, other):
        if other is None:
            return False
        if not isinstance(other, ToolkitLinkedStagedSource):
            return False
        return super(ToolkitLinkedStagedSource, self).__eq__(other) and \
               self.stop_staging == other.stop_staging and \
               self.start_staging == other.start_staging and \
               self.resync == other.resync and \
               self.pre_snapshot == other.pre_snapshot and \
               self.post_snapshot == other.post_snapshot and \
               self.status_script == other.status_script

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        return id(self)

    def __repr__(self):
        return common.generate_repr_string(self)

    @property
    def stop_staging(self):
        """
        A workflow script that stop the staged source. The staged files will be
        mounted and available. Upon completion of this workflow, the staged
        files will be unmounted.

        :rtype: ``TEXT_TYPE``
        """
        return self._stop_staging[0]

    @stop_staging.setter
    def stop_staging(self, value):
        self._stop_staging = (value, True)

    @property
    def start_staging(self):
        """
        A workflow script that start the staged source. The staged files will
        be mounted and available.

        :rtype: ``TEXT_TYPE``
        """
        return self._start_staging[0]

    @start_staging.setter
    def start_staging(self, value):
        self._start_staging = (value, True)

    @property
    def resync(self):
        """
        A workflow script that builds the staging instance from production.

        :rtype: ``TEXT_TYPE``
        """
        return self._resync[0]

    @resync.setter
    def resync(self, value):
        self._resync = (value, True)

    @property
    def pre_snapshot(self):
        """
        A workflow script to run just prior to snapshotting the staged source.

        :rtype: ``TEXT_TYPE``
        """
        return self._pre_snapshot[0]

    @pre_snapshot.setter
    def pre_snapshot(self, value):
        self._pre_snapshot = (value, True)

    @property
    def post_snapshot(self):
        """
        A workflow script to run immediately after snapshotting the staged
        source.

        :rtype: ``TEXT_TYPE``
        """
        return self._post_snapshot[0]

    @post_snapshot.setter
    def post_snapshot(self, value):
        self._post_snapshot = (value, True)

    @property
    def status_script(self):
        """
        A script run on the staging host to determine if the staging instance
        is configured to receive data from the dSource. The script should exit
        with an exit status of 0 if data reception is correctly configured and
        available. The script should exit with an exit status of 128 if the
        staging instance is not configured to receive data. Any other exit
        status implies the script encountered an unexpected state or error. If
        no status script is supplied, the dSource will always be in an active
        state while enabled.

        :rtype: ``TEXT_TYPE``
        """
        return self._status_script[0]

    @status_script.setter
    def status_script(self, value):
        self._status_script = (value, True)




#
# Copyright 2019 by Delphix
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

#
# This class has been automatically generated from:
#     /delphix-oracle-virtual-source.json
#
# Do not edit this file manually!
#

from __future__ import unicode_literals

from delphixpy.v1_9_1.web.objects.OracleSource import OracleSource
from delphixpy.v1_9_1 import factory
from delphixpy.v1_9_1 import common

class __Undef(object):
    def __repr__(self):
        return "undef"

    def __setattr__(self, name, value):
        raise Exception('Cannot modify attributes of __Undef.')

_UNDEFINED = __Undef()

try:
    TEXT_TYPE = unicode
except NameError:
    TEXT_TYPE = str

class OracleVirtualSource(OracleSource):
    """
    *(extends* :py:class:`v1_9_1.web.vo.OracleSource` *)* A virtual Oracle
    source.
    """
    def __init__(self, undef_enabled=True):
        super(OracleVirtualSource, self).__init__()
        self._type = ("OracleVirtualSource", True)
        self._operations = (self.__undef__, True)
        self._config_params = (self.__undef__, True)
        self._node_listeners = (self.__undef__, True)
        self._redo_log_groups = (self.__undef__, True)
        self._file_mapping_rules = (self.__undef__, True)
        self._mount_base = (self.__undef__, True)
        self._redo_log_size_in_mb = (self.__undef__, True)
        self._archivelog_mode = (self.__undef__, True)
        self._custom_env_vars = (self.__undef__, True)
        self._allow_auto_vdb_restart_on_host_reboot = (self.__undef__, True)
        self._config_template = (self.__undef__, True)

    API_VERSION = "1.9.1"

    @classmethod
    def from_dict(cls, data, dirty=False, undef_enabled=True):
        obj = super(OracleVirtualSource, cls).from_dict(data, dirty, undef_enabled)
        if "operations" in data and data["operations"] is not None:
            obj._operations = (factory.create_object(data["operations"], "VirtualSourceOperations"), dirty)
            factory.validate_type(obj._operations[0], "VirtualSourceOperations")
        else:
            obj._operations = (obj.__undef__, dirty)
        obj._config_params = (data.get("configParams", obj.__undef__), dirty)
        if obj._config_params[0] is not None and obj._config_params[0] is not obj.__undef__:
            assert isinstance(obj._config_params[0], dict), ("Expected one of [u'object'], but got %s of type %s" % (obj._config_params[0], type(obj._config_params[0])))
            common.validate_format(obj._config_params[0], "None", None, None)
        obj._node_listeners = []
        for item in data.get("nodeListeners") or []:
            assert isinstance(item, TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (item, type(item)))
            common.validate_format(item, "objectReference", None, None)
            obj._node_listeners.append(item)
        obj._node_listeners = (obj._node_listeners, dirty)
        obj._redo_log_groups = (data.get("redoLogGroups", obj.__undef__), dirty)
        if obj._redo_log_groups[0] is not None and obj._redo_log_groups[0] is not obj.__undef__:
            assert isinstance(obj._redo_log_groups[0], int), ("Expected one of [u'integer'], but got %s of type %s" % (obj._redo_log_groups[0], type(obj._redo_log_groups[0])))
            common.validate_format(obj._redo_log_groups[0], "None", None, None)
        obj._file_mapping_rules = (data.get("fileMappingRules", obj.__undef__), dirty)
        if obj._file_mapping_rules[0] is not None and obj._file_mapping_rules[0] is not obj.__undef__:
            assert isinstance(obj._file_mapping_rules[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._file_mapping_rules[0], type(obj._file_mapping_rules[0])))
            common.validate_format(obj._file_mapping_rules[0], "None", None, None)
        obj._mount_base = (data.get("mountBase", obj.__undef__), dirty)
        if obj._mount_base[0] is not None and obj._mount_base[0] is not obj.__undef__:
            assert isinstance(obj._mount_base[0], TEXT_TYPE), ("Expected one of [u'string'], but got %s of type %s" % (obj._mount_base[0], type(obj._mount_base[0])))
            common.validate_format(obj._mount_base[0], "None", None, 256)
        obj._redo_log_size_in_mb = (data.get("redoLogSizeInMB", obj.__undef__), dirty)
        if obj._redo_log_size_in_mb[0] is not None and obj._redo_log_size_in_mb[0] is not obj.__undef__:
            assert isinstance(obj._redo_log_size_in_mb[0], int), ("Expected one of [u'integer'], but got %s of type %s" % (obj._redo_log_size_in_mb[0], type(obj._redo_log_size_in_mb[0])))
            common.validate_format(obj._redo_log_size_in_mb[0], "None", None, None)
        obj._archivelog_mode = (data.get("archivelogMode", obj.__undef__), dirty)
        if obj._archivelog_mode[0] is not None and obj._archivelog_mode[0] is not obj.__undef__:
            assert isinstance(obj._archivelog_mode[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._archivelog_mode[0], type(obj._archivelog_mode[0])))
            common.validate_format(obj._archivelog_mode[0], "None", None, None)
        obj._custom_env_vars = []
        for item in data.get("customEnvVars") or []:
            obj._custom_env_vars.append(factory.create_object(item))
            factory.validate_type(obj._custom_env_vars[-1], "OracleCustomEnvVar")
        obj._custom_env_vars = (obj._custom_env_vars, dirty)
        obj._allow_auto_vdb_restart_on_host_reboot = (data.get("allowAutoVDBRestartOnHostReboot", obj.__undef__), dirty)
        if obj._allow_auto_vdb_restart_on_host_reboot[0] is not None and obj._allow_auto_vdb_restart_on_host_reboot[0] is not obj.__undef__:
            assert isinstance(obj._allow_auto_vdb_restart_on_host_reboot[0], bool), ("Expected one of [u'boolean'], but got %s of type %s" % (obj._allow_auto_vdb_restart_on_host_reboot[0], type(obj._allow_auto_vdb_restart_on_host_reboot[0])))
            common.validate_format(obj._allow_auto_vdb_restart_on_host_reboot[0], "None", None, None)
        obj._config_template = (data.get("configTemplate", obj.__undef__), dirty)
        if obj._config_template[0] is not None and obj._config_template[0] is not obj.__undef__:
            assert isinstance(obj._config_template[0], TEXT_TYPE), ("Expected one of [u'string', u'null'], but got %s of type %s" % (obj._config_template[0], type(obj._config_template[0])))
            common.validate_format(obj._config_template[0], "objectReference", None, None)
        return obj

    def to_dict(self, dirty=False, belongs_to_parent=False):
        dct = super(OracleVirtualSource, self).to_dict(dirty, belongs_to_parent)

        def dictify(obj, prop_is_list_or_vo=False):
            if isinstance(obj, list):
                return [dictify(o, prop_is_list_or_vo) for o in obj]
            elif hasattr(obj, "to_dict"):
                return obj.to_dict(dirty=dirty, belongs_to_parent=prop_is_list_or_vo)
            else:
                return obj
        if "operations" == "type" or (self.operations is not self.__undef__ and (not (dirty and not self._operations[1]) or isinstance(self.operations, list) or belongs_to_parent)):
            dct["operations"] = dictify(self.operations, prop_is_list_or_vo=True)
        if "config_params" == "type" or (self.config_params is not self.__undef__ and (not (dirty and not self._config_params[1]) or isinstance(self.config_params, list) or belongs_to_parent)):
            dct["configParams"] = dictify(self.config_params, prop_is_list_or_vo=True)
        if "node_listeners" == "type" or (self.node_listeners is not self.__undef__ and (not (dirty and not self._node_listeners[1]) or isinstance(self.node_listeners, list) or belongs_to_parent)):
            dct["nodeListeners"] = dictify(self.node_listeners, prop_is_list_or_vo=True)
        if "redo_log_groups" == "type" or (self.redo_log_groups is not self.__undef__ and (not (dirty and not self._redo_log_groups[1]) or isinstance(self.redo_log_groups, list) or belongs_to_parent)):
            dct["redoLogGroups"] = dictify(self.redo_log_groups)
        elif belongs_to_parent and self.redo_log_groups is self.__undef__:
            dct["redoLogGroups"] = 3
        if "file_mapping_rules" == "type" or (self.file_mapping_rules is not self.__undef__ and (not (dirty and not self._file_mapping_rules[1]) or isinstance(self.file_mapping_rules, list) or belongs_to_parent)):
            dct["fileMappingRules"] = dictify(self.file_mapping_rules)
        if "mount_base" == "type" or (self.mount_base is not self.__undef__ and (not (dirty and not self._mount_base[1]) or isinstance(self.mount_base, list) or belongs_to_parent)):
            dct["mountBase"] = dictify(self.mount_base)
        if "redo_log_size_in_mb" == "type" or (self.redo_log_size_in_mb is not self.__undef__ and (not (dirty and not self._redo_log_size_in_mb[1]) or isinstance(self.redo_log_size_in_mb, list) or belongs_to_parent)):
            dct["redoLogSizeInMB"] = dictify(self.redo_log_size_in_mb)
        if "archivelog_mode" == "type" or (self.archivelog_mode is not self.__undef__ and (not (dirty and not self._archivelog_mode[1]) or isinstance(self.archivelog_mode, list) or belongs_to_parent)):
            dct["archivelogMode"] = dictify(self.archivelog_mode)
        elif belongs_to_parent and self.archivelog_mode is self.__undef__:
            dct["archivelogMode"] = True
        if "custom_env_vars" == "type" or (self.custom_env_vars is not self.__undef__ and (not (dirty and not self._custom_env_vars[1]) or isinstance(self.custom_env_vars, list) or belongs_to_parent)):
            dct["customEnvVars"] = dictify(self.custom_env_vars, prop_is_list_or_vo=True)
        if "allow_auto_vdb_restart_on_host_reboot" == "type" or (self.allow_auto_vdb_restart_on_host_reboot is not self.__undef__ and (not (dirty and not self._allow_auto_vdb_restart_on_host_reboot[1]) or isinstance(self.allow_auto_vdb_restart_on_host_reboot, list) or belongs_to_parent)):
            dct["allowAutoVDBRestartOnHostReboot"] = dictify(self.allow_auto_vdb_restart_on_host_reboot)
        if "config_template" == "type" or (self.config_template is not self.__undef__ and (not (dirty and not self._config_template[1]) or isinstance(self.config_template, list) or belongs_to_parent)):
            dct["configTemplate"] = dictify(self.config_template)
        return dct

    def dirty(self):
        return self.from_dict(self.to_dict(dirty=False), dirty=True)

    def force_dirty(self):
        self._operations = (self._operations[0], True)
        self._config_params = (self._config_params[0], True)
        self._node_listeners = (self._node_listeners[0], True)
        self._redo_log_groups = (self._redo_log_groups[0], True)
        self._file_mapping_rules = (self._file_mapping_rules[0], True)
        self._mount_base = (self._mount_base[0], True)
        self._redo_log_size_in_mb = (self._redo_log_size_in_mb[0], True)
        self._archivelog_mode = (self._archivelog_mode[0], True)
        self._custom_env_vars = (self._custom_env_vars[0], True)
        self._allow_auto_vdb_restart_on_host_reboot = (self._allow_auto_vdb_restart_on_host_reboot[0], True)
        self._config_template = (self._config_template[0], True)

    def is_dirty(self):
        return any([self._operations[1], self._config_params[1], self._node_listeners[1], self._redo_log_groups[1], self._file_mapping_rules[1], self._mount_base[1], self._redo_log_size_in_mb[1], self._archivelog_mode[1], self._custom_env_vars[1], self._allow_auto_vdb_restart_on_host_reboot[1], self._config_template[1]])

    def __eq__(self, other):
        if other is None:
            return False
        if not isinstance(other, OracleVirtualSource):
            return False
        return super(OracleVirtualSource, self).__eq__(other) and \
               self.operations == other.operations and \
               self.config_params == other.config_params and \
               self.node_listeners == other.node_listeners and \
               self.redo_log_groups == other.redo_log_groups and \
               self.file_mapping_rules == other.file_mapping_rules and \
               self.mount_base == other.mount_base and \
               self.redo_log_size_in_mb == other.redo_log_size_in_mb and \
               self.archivelog_mode == other.archivelog_mode and \
               self.custom_env_vars == other.custom_env_vars and \
               self.allow_auto_vdb_restart_on_host_reboot == other.allow_auto_vdb_restart_on_host_reboot and \
               self.config_template == other.config_template

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        return id(self)

    def __repr__(self):
        return common.generate_repr_string(self)

    @property
    def operations(self):
        """
        User-specified operation hooks for this source.

        :rtype: :py:class:`v1_9_1.web.vo.VirtualSourceOperations`
        """
        return self._operations[0]

    @operations.setter
    def operations(self, value):
        self._operations = (value, True)

    @property
    def config_params(self):
        """
        Oracle database configuration parameter overrides.

        :rtype: ``dict``
        """
        return self._config_params[0]

    @config_params.setter
    def config_params(self, value):
        self._config_params = (value, True)

    @property
    def node_listeners(self):
        """
        A list of object references to Oracle Node Listeners selected for this
        Virtual Database. Delphix picks one default listener from the target
        environment if this list is empty at virtual database provision time.

        :rtype: ``list`` of ``TEXT_TYPE``
        """
        return self._node_listeners[0]

    @node_listeners.setter
    def node_listeners(self, value):
        self._node_listeners = (value, True)

    @property
    def redo_log_groups(self):
        """
        *(default value: 3)* Number of Online Redo Log Groups.

        :rtype: ``int``
        """
        return self._redo_log_groups[0]

    @redo_log_groups.setter
    def redo_log_groups(self, value):
        self._redo_log_groups = (value, True)

    @property
    def file_mapping_rules(self):
        """
        Database file mapping rules.

        :rtype: ``TEXT_TYPE``
        """
        return self._file_mapping_rules[0]

    @file_mapping_rules.setter
    def file_mapping_rules(self, value):
        self._file_mapping_rules = (value, True)

    @property
    def mount_base(self):
        """
        The base mount point to use for the NFS mounts.

        :rtype: ``TEXT_TYPE``
        """
        return self._mount_base[0]

    @mount_base.setter
    def mount_base(self, value):
        self._mount_base = (value, True)

    @property
    def redo_log_size_in_mb(self):
        """
        Online Redo Log size in MB.

        :rtype: ``int``
        """
        return self._redo_log_size_in_mb[0]

    @redo_log_size_in_mb.setter
    def redo_log_size_in_mb(self, value):
        self._redo_log_size_in_mb = (value, True)

    @property
    def archivelog_mode(self):
        """
        *(default value: True)* Archive Log Mode of the Oracle virtual
        database.

        :rtype: ``bool``
        """
        return self._archivelog_mode[0]

    @archivelog_mode.setter
    def archivelog_mode(self, value):
        self._archivelog_mode = (value, True)

    @property
    def custom_env_vars(self):
        """
        Custom environment variables for Oracle databases.

        :rtype: ``list`` of :py:class:`v1_9_1.web.vo.OracleCustomEnvVar`
        """
        return self._custom_env_vars[0]

    @custom_env_vars.setter
    def custom_env_vars(self, value):
        self._custom_env_vars = (value, True)

    @property
    def allow_auto_vdb_restart_on_host_reboot(self):
        """
        Indicates whether Delphix should automatically restart this virtual
        source when target host reboot is detected.

        :rtype: ``bool``
        """
        return self._allow_auto_vdb_restart_on_host_reboot[0]

    @allow_auto_vdb_restart_on_host_reboot.setter
    def allow_auto_vdb_restart_on_host_reboot(self, value):
        self._allow_auto_vdb_restart_on_host_reboot = (value, True)

    @property
    def config_template(self):
        """
        Optional database template to use for provisioning and refresh. If set,
        configParams will be ignored on provision or refresh.

        :rtype: ``TEXT_TYPE`` *or* ``null``
        """
        return self._config_template[0]

    @config_template.setter
    def config_template(self, value):
        self._config_template = (value, True)

